@@ -27,30 +27,32 @@
 #  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 #  THE SOFTWARE.
 
-
 """GUI tool for configuring azimuthal integration on series of files."""
 
 __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "28/02/2019"
+__date__ = "07/01/2022"
 __satus__ = "production"
 
 import sys
-import logging
 import time
 import numpy
 import os.path
 import collections
 import contextlib
-import six
-
-import fabio
-
+from argparse import ArgumentParser
+import logging
 logging.basicConfig(level=logging.INFO)
 logging.captureWarnings(True)
-logger = logging.getLogger("pyFAI")
+logger = logging.getLogger(__name__)
+try:
+    import hdf5plugin  # noqa
+except ImportError:
+    logger.debug("Unable to load hdf5plugin, backtrace:", exc_info=True)
+
+import fabio
 
 import pyFAI.utils
 import pyFAI.worker
@@ -61,8 +63,6 @@ from pyFAI.utils.shell import ProgressBar
 from pyFAI.utils import logging_utils
 from pyFAI.utils import header_utils
 
-from argparse import ArgumentParser
-
 try:
     from rfoo.utils import rconsole
     rconsole.spawn_server()
@@ -108,6 +108,7 @@ def integrate_gui(options, args):
             builder.addImageFormat("NumPy binary files", "npy")
             builder.addImageFormat("CBF files", "cbf")
             builder.addImageFormat("MarCCD image files", "mccd")
+            builder.addImageFormat("HDF5 image stack", "h5 hdf5 nxs")
             dialog.setNameFilters(builder.getFilters())
 
             dialog.setFileMode(qt.QFileDialog.ExistingFiles)
@@ -126,9 +127,10 @@ def integrate_gui(options, args):
         moveCenterTo(dialog, center)
 
         class QtProcess(qt.QThread):
+
             def run(self):
                 observer = dialog.createObserver(qtSafe=True)
-                process(input_data, window.output_path, config, options.monitor_key, observer, options.write_mode)
+                process(input_data, window.output_path, config, options.monitor_key, observer, options.write_mode, format_=options.format.lower())
 
         qtProcess = QtProcess()
         qtProcess.start()
@@ -345,7 +347,7 @@ class DataSource(object):
             return False
 
         item = self._items[0]
-        if isinstance(item, six.string_types):
+        if isinstance(item, (str,)):
             with fabio.open(item) as fabio_image:
                 multiframe = fabio_image.nframes > 1
         elif isinstance(item, fabio.fabioimage.FabioImage):
@@ -366,7 +368,7 @@ class DataSource(object):
         return count > 0
 
     def _iter_item_frames(self, iitem, start_id, item):
-        if isinstance(item, six.string_types):
+        if isinstance(item, (str,)):
             with self._statistics.time_reading():
                 fabio_image = fabio.open(item)
             filename = fabio_image.filename
@@ -528,17 +530,17 @@ class Statistics(object):
         self._execution = 0
 
     def execution_started(self):
-        self._start_time = time.time()
+        self._start_time = time.perf_counter()
 
     def execution_finished(self):
-        t = time.time()
+        t = time.perf_counter()
         self._execution = t - self._start_time
 
     @contextlib.contextmanager
     def time_processing(self):
-        t1 = time.time()
+        t1 = time.perf_counter()
         yield
-        t2 = time.time()
+        t2 = time.perf_counter()
         processing = t2 - t1
         if self._processing == 0:
             self._first_processing = processing
@@ -547,9 +549,9 @@ class Statistics(object):
 
     @contextlib.contextmanager
     def time_reading(self):
-        t1 = time.time()
+        t1 = time.perf_counter()
         yield
-        t2 = time.time()
+        t2 = time.perf_counter()
         reading = t2 - t1
         self._reading += reading
 
@@ -581,7 +583,7 @@ class Statistics(object):
         return self._execution
 
 
-def process(input_data, output, config, monitor_name, observer, write_mode=HDF5Writer.MODE_ERROR):
+def process(input_data, output, config, monitor_name, observer, write_mode=HDF5Writer.MODE_ERROR, format_=None):
     """
     Integrate a set of data.
 
@@ -590,6 +592,7 @@ def process(input_data, output, config, monitor_name, observer, write_mode=HDF5W
     :param dict config: Dictionary to configure `pyFAI.worker.Worker`
     :param IntegrationObserver observer: Observer of the processing
     :param str write_mode: Specify options to deal with IO errors
+    :param format_: output format
     """
     statistics = Statistics()
     statistics.execution_started()
@@ -623,7 +626,7 @@ def process(input_data, output, config, monitor_name, observer, write_mode=HDF5W
     # Skip invalide data
     source = DataSource(statistics=statistics)
     for item in input_data:
-        if isinstance(item, six.string_types):
+        if isinstance(item, (str,)):
             if os.path.isfile(item):
                 source.append(item)
             else:
@@ -655,7 +658,7 @@ def process(input_data, output, config, monitor_name, observer, write_mode=HDF5W
             entry_path = None
         if os.path.isdir(output):
             writer = MultiFileWriter(output, mode=write_mode)
-        elif output.endswith(".h5") or output.endswith(".hdf5"):
+        elif output.endswith(".h5") or output.endswith(".hdf5") or format_ in ("h5", "hdf5"):
             writer = HDF5Writer(output, hpath=entry_path, append_frames=True, mode=write_mode)
         else:
             output_path = os.path.abspath(output)
@@ -698,6 +701,15 @@ def process(input_data, output, config, monitor_name, observer, write_mode=HDF5W
             result = worker.process(data=data_info.data,
                                     normalization_factor=normalization_factor,
                                     writer=writer)
+        # Store reference to input data if possible
+        if isinstance(writer, HDF5Writer) and (data_info.fabio_image is not None):
+            fimg = data_info.fabio_image
+            if "dataset" in dir(fimg):
+                if isinstance(fimg.dataset, list):
+                    for ds in  fimg.dataset:
+                        writer.set_hdf5_input_dataset(ds)
+                else:
+                    writer.set_hdf5_input_dataset(fimg.dataset)
 
         if observer.is_interruption_requested():
             break
@@ -775,8 +787,8 @@ http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=697348"""
                         dest="output", default=None,
                         help="Directory or file where to store the output data")
     parser.add_argument("-f", "--format",
-                        dest="format", default=None,
-                        help="output data format (can be HDF5)")
+                        dest="format", default="None",
+                        help="output data format (can be used to enforce HDF5 in combination with --output)")
     parser.add_argument("-s", "--slow-motor",
                         dest="slow", default=None,
                         help="Dimension of the scan on the slow direction (makes sense only with HDF5)")
