@@ -3,7 +3,7 @@
 #    Project: Azimuthal integration
 #             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2015-2018 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2015-2022 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -25,38 +25,33 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-from __future__ import absolute_import, print_function, division
-
 """Module with GUI for diffraction mapping experiments"""
 
-__author__ = "Jerome Kieffer"
+__author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "24/01/2020"
+__date__ = "01/02/2022"
 __status__ = "development"
 __docformat__ = 'restructuredtext'
 
 import os
 import time
-import posixpath
-import sys
 import collections
 import glob
+from argparse import ArgumentParser
+from urllib.parse import urlparse
 import logging
 logger = logging.getLogger(__name__)
 import numpy
 import fabio
 import json
-
+import __main__ as main
 from .opencl import ocl
-from .units import to_unit
-from .third_party import six
 from . import version as PyFAI_VERSION, date as PyFAI_DATE, load
-from .io import Nexus, get_isotime
+from .io import Nexus, get_isotime, h5py
 from .worker import Worker, _reduce_images
-from argparse import ArgumentParser
-urlparse = six.moves.urllib.parse.urlparse
+from .containers import Integrate1dResult, Integrate2dResult
 
 DIGITS = [str(i) for i in range(10)]
 Position = collections.namedtuple('Position', 'index, rot, trans')
@@ -66,6 +61,7 @@ class DiffMap(object):
     """
     Basic class for diffraction mapping experiment using pyFAI
     """
+
     def __init__(self, npt_fast=0, npt_slow=1, npt_rad=1000, npt_azim=None):
         """Constructor of the class DiffMap for diffraction mapping
 
@@ -77,34 +73,35 @@ class DiffMap(object):
         self.npt_fast = npt_fast
         self.npt_slow = npt_slow
         self.npt_rad = npt_rad
+        self.npt_azim = npt_azim
         self.slow_motor_name = "slow"
         self.fast_motor_name = "fast"
         self.offset = 0
         self.poni = None
         self.worker = Worker(unit="2th_deg")
-        self.worker.output = "raw" # exchange IntegrateResults, not numpy arrays
+        self.worker.output = "raw"  # exchange IntegrateResults, not numpy arrays
         self.dark = None
         self.flat = None
         self.mask = None
         self.I0 = None
         self.hdf5 = None
-        self.hdf5path = "diff_map/data/map"
-        self.group = None
+        self.nxdata_grp = None
         self.dataset = None
+        self.dataset_error = None
         self.inputfiles = []
         self.timing = []
         self.stats = False
         self._idx = -1
         self.processed_file = []
+        self.stored_input = set()
         self.nxs = None
+        self.entry_grp = None
         self.experiment_title = "Diffraction Mapping"
 
     def __repr__(self):
         return "%s experiment with ntp_slow: %s ntp_fast: %s, npt_diff: %s" % \
             (self.experiment_title, self.npt_slow, self.npt_fast, self.npt_rad)
 
-
-    
     @staticmethod
     def to_tuple(name):
         """
@@ -175,6 +172,9 @@ If the number of files is too large, use double quotes like "*.edf" """
         parser.add_argument("-c", "--npt", dest="npt_rad",
                             help="number of points in diffraction powder pattern. Mandatory without GUI",
                             default=None)
+        parser.add_argument( "--npt-azim", dest="npt_azim",
+                            help="number of points in azimuthal direction, 1 for 1D integration",
+                            default=None)
         parser.add_argument("-d", "--dark", dest="dark", metavar="FILE",
                             help="list of dark images to average and subtract (comma separated list)",
                             default=None)
@@ -273,8 +273,11 @@ If the number of files is too large, use double quotes like "*.edf" """
             self.npt_slow = int(options.slow)
             config["slow_motor_points"] = self.npt_slow
         if options.npt_rad is not None:
-            self.npt_rad = int(options.npt_rad)
-            config["ai"]["nbpt_rad"] = self.npt_rad,
+            config["ai"]["nbpt_rad"] = self.npt_rad = int(options.npt_rad)
+            # Why was config["ai"]["nbpt_rad"] a tuple ?
+        if options.npt_azim is not None:
+            config["ai"]["nbpt_azim"] = self.npt_azim = int(options.npt_azim)
+
         if options.offset is not None:
             self.offset = int(options.offset)
             config["offset"] = self.offset,
@@ -302,8 +305,9 @@ If the number of files is too large, use double quotes like "*.edf" """
         """
         Create the HDF5 structure if needed ...
         """
-        import h5py
-        dtype = h5py.special_dtype(vlen=six.text_type)
+        if h5py is None:
+            raise RuntimeError("h5py is needed to create HDF5 files")
+        dtype = h5py.special_dtype(vlen=str)
 
         if self.hdf5 is None:
             raise RuntimeError("No output HDF5 file provided")
@@ -312,56 +316,64 @@ If the number of files is too large, use double quotes like "*.edf" """
         if os.path.exists(self.hdf5) and rewrite:
             os.unlink(self.hdf5)
 
-        spath = self.hdf5path.split("/")
-        assert len(spath) > 2
-        nxs = Nexus(self.hdf5, mode="w")
-        entry = nxs.new_entry(entry=spath[0], program_name="pyFAI", title="diffmap")
-        grp = entry
-        for subgrp in spath[1:-2]:
-            entry.attrs["default"] = subgrp
-            grp = nxs.new_class(grp, name=subgrp, class_type="NXcollection")
+        nxs = Nexus(self.hdf5, mode="w", creator="pyFAI")
+        self.entry_grp = entry_grp = nxs.new_entry(entry="entry",
+                                                   program_name="pyFAI",
+                                                   title="diff_map")
 
-        processgrp = nxs.new_class(grp, "pyFAI", class_type="NXprocess")
-        processgrp["program"] = numpy.array([i for i in sys.argv], dtype=dtype)
-        processgrp["version"] = PyFAI_VERSION
-        processgrp["date"] = get_isotime()
+        process_grp = nxs.new_class(entry_grp, "pyFAI", class_type="NXprocess")
+        process_grp["program"] = main.__file__
+        process_grp["version"] = PyFAI_VERSION
+        process_grp["date"] = get_isotime()
         if self.mask:
-            processgrp["maskfile"] = self.mask
+            process_grp["maskfile"] = self.mask
         if self.flat:
-            processgrp["flatfiles"] = numpy.array([i for i in self.flat], dtype=dtype)
+            process_grp["flatfiles"] = numpy.array([i for i in self.flat], dtype=dtype)
         if self.dark:
-            processgrp["darkfiles"] = numpy.array([i for i in self.dark], dtype=dtype)
-        processgrp["inputfiles"] = numpy.array([i for i in self.inputfiles], dtype=dtype)
+            process_grp["darkfiles"] = numpy.array([i for i in self.dark], dtype=dtype)
         if self.poni is not None:
-            processgrp["PONIfile"] = self.poni
+            process_grp["PONIfile"] = self.poni
+        process_grp["inputfiles"] = numpy.array([i for i in self.inputfiles], dtype=dtype)
 
-        processgrp["dim0"] = self.npt_slow
-        processgrp["dim0"].attrs["axis"] = self.slow_motor_name
-        processgrp["dim1"] = self.npt_fast
-        processgrp["dim1"].attrs["axis"] = self.fast_motor_name
-        processgrp["dim2"] = self.npt_rad
-        processgrp["dim2"].attrs["axis"] = "diffraction"
-        for k, v in self.ai.getPyFAI().items():
-            processgrp[k] = v
+        process_grp["dim0"] = self.npt_slow
+        process_grp["dim0"].attrs["axis"] = self.slow_motor_name
+        process_grp["dim1"] = self.npt_fast
+        process_grp["dim1"].attrs["axis"] = self.fast_motor_name
+        process_grp["dim2"] = self.npt_rad
+        process_grp["dim2"].attrs["axis"] = "diffraction"
+        config = nxs.new_class(process_grp, "configuration", "NXnote")
+        config["type"] = "text/json"
+        worker_config = self.worker.get_config()
+        config["data"] = json.dumps(worker_config, indent=2, separators=(",\r\n", ": "))
+        print(worker_config)
 
-        nxdataName = spath[-2]
-        self.group = nxs.new_class(grp, name=nxdataName, class_type="NXdata")
-        grp.attrs["default"] = nxdataName
-
-        if posixpath.basename(self.hdf5path) in self.group:
-            self.dataset = self.group[posixpath.basename(self.hdf5path)]
+        self.nxdata_grp = nxs.new_class(process_grp, "result", class_type="NXdata")
+        entry_grp.attrs["default"] = self.nxdata_grp.name.split("/", 2)[2]
+        
+        if self.worker.do_2D():
+            self.dataset = self.nxdata_grp.create_dataset(
+                            name="intensity",
+                            shape=(self.npt_slow, self.npt_fast, self.npt_azim, self.npt_rad),
+                            dtype="float32",
+                            chunks=(1, 1, self.npt_azim, self.npt_rad),
+                            maxshape=(None, None, self.npt_azim, self.npt_rad),
+                            fillvalue=numpy.NaN)
+            self.dataset.attrs["interpretation"] = "image"
+            self.nxdata_grp.attrs["axes"] = [".", ".", "azimuthal", str(self.unit).split("_")[0]]
         else:
-            self.dataset = self.group.create_dataset(
-                name=posixpath.basename(self.hdf5path),
-                shape=(self.npt_slow, self.npt_fast, self.npt_rad),
-                dtype="float32",
-                chunks=(1, self.npt_fast, self.npt_rad),
-                maxshape=(None, None, self.npt_rad))
-            self.dataset.attrs["signal"] = "1"
+            self.dataset = self.nxdata_grp.create_dataset(
+                            name="intensity",
+                            shape=(self.npt_slow, self.npt_fast, self.npt_rad),
+                            dtype="float32",
+                            chunks=(1, self.npt_fast, self.npt_rad),
+                            maxshape=(None, None, self.npt_rad),
+                            fillvalue=numpy.NaN)
             self.dataset.attrs["interpretation"] = "spectrum"
-            self.dataset.attrs["axes"] = str(self.unit).split("_")[0]
-            self.dataset.attrs["creator"] = "pyFAI"
-            self.dataset.attrs["long_name"] = str(self)
+            self.nxdata_grp.attrs["axes"] = [".", ".", str(self.unit).split("_")[0]]    
+
+        self.nxdata_grp.attrs["signal"] = self.dataset.name.split("/")[-1]
+        
+        self.dataset.attrs["title"] = str(self)
         self.nxs = nxs
 
     def setup_ai(self):
@@ -382,11 +394,11 @@ If the number of files is too large, use double quotes like "*.edf" """
     def init_ai(self):
         """Force initialization of azimuthal intgrator
 
-        :return: radial position array
+        :return: radial and azimuthal position arrays
         """
         if not self.ai:
             self.setup_ai()
-        if not self.group:
+        if not self.nxdata_grp:
             self.makeHDF5(rewrite=False)
         if self.ai.detector.shape:
             # shape of detector undefined: reading the first image to guess it
@@ -397,21 +409,36 @@ If the number of files is too large, use double quotes like "*.edf" """
         self.worker.shape = shape
         self.worker.output = "raw"
         data = numpy.empty(shape, dtype=numpy.float32)
-        print("Initialization of the Azimuthal Integrator using method %s" % (self.method, ))
+        print(f"Initialization of the Azimuthal Integrator using method {self.method}")
         # enforce initialization of azimuthal integrator
         print(self.ai)
         res = self.worker.process(data)
         tth = res.radial
         if self.dataset is None:
             self.makeHDF5()
+
+        if res.sigma is not None:
+            self.dataset_error = self.nxdata_grp.create_dataset("errors",
+                                                                shape=self.dataset.shape,
+                                                                dtype="float32",
+                                                                chunks=(1,) + self.dataset.shape[1:],
+                                                                maxshape=(None,) + self.dataset.shape[1:])
+            self.dataset_error.attrs["interpretation"] = "image" if self.dataset.ndim==4 else "spectrum"
         space, unit = str(self.unit).split("_")
-        if space not in self.group:
-            self.group[space] = tth
-            self.group[space].attrs["axes"] = 3
-            self.group[space].attrs["unit"] = unit
-            self.group[space].attrs["long_name"] = self.unit.label
-            self.group[space].attrs["interpretation"] = "scalar"
-        return tth
+        if space not in self.nxdata_grp:
+            self.nxdata_grp[space] = tth
+            self.nxdata_grp[space].attrs["axes"] = 3
+            self.nxdata_grp[space].attrs["unit"] = unit
+            self.nxdata_grp[space].attrs["long_name"] = self.unit.label
+            self.nxdata_grp[space].attrs["interpretation"] = "scalar"
+        if self.worker.do_2D():
+            self.nxdata_grp["azimuthal"] = res.azimuthal
+            self.nxdata_grp["azimuthal"].attrs["unit"] = "deg"
+            self.nxdata_grp["azimuthal"].attrs["interpretation"] = "scalar"
+            azim = res.azimuthal
+        else:
+            azim = None
+        return tth, azim
 
     def show_stats(self):
         if not self.stats:
@@ -429,7 +456,7 @@ If the number of files is too large, use double quotes like "*.edf" """
         ax.set_title("Execution time")
         ax.grid(True)
         fig.show()
-        six.moves.input("Enter to quit")
+        input("Enter to quit")
 
     def get_pos(self, filename=None, idx=None):
         """
@@ -440,7 +467,6 @@ If the number of files is too large, use double quotes like "*.edf" """
         :param idx: index of current frame
         :return: namedtuple: index, rot, trans
         """
-        #         n = int(filename.split(".")[0].split("_")[-1]) - (self.offset or 0)
         if idx is None:
             n = self.inputfiles.index(filename) - self.offset
         else:
@@ -457,19 +483,48 @@ If the number of files is too large, use double quotes like "*.edf" """
         if self.dataset is None:
             self.makeHDF5()
 
-        t = time.time()
+        t = time.perf_counter()
         fimg = fabio.open(filename)
+        if "dataset" in dir(fimg):
+            if isinstance(fimg.dataset, list):
+                for ds in fimg.dataset:
+                    self.set_hdf5_input_dataset(ds)
+            else:
+                self.set_hdf5_input_dataset(fimg.dataset)
         self.process_one_frame(fimg.data)
         if fimg.nframes > 1:
             for i in range(fimg.nframes - 1):
                 fimg = fimg.next()
                 self.process_one_frame(fimg.data)
-        t -= time.time()
+        t -= time.perf_counter()
         print("Processing %30s took %6.1fms (%i frames)" %
               (os.path.basename(filename), -1000.0 * t, fimg.nframes))
         self.timing.append(-t)
         self.processed_file.append(filename)
 
+    def set_hdf5_input_dataset(self, dataset):
+        "record the input dataset with an external link"
+        if not isinstance(dataset, h5py.Dataset):
+            return
+        if not (self.nxs and self.nxs.h5 and self.entry_grp):
+            return
+        id_ = id(dataset)
+        if id_ in self.stored_input:
+            return
+        else:
+            self.stored_input.add(id_)
+        # Process 0: measurement group
+        if "measurement" in self.entry_grp:
+            measurement_grp = self.entry_grp["measurement"]
+        else:
+            measurement_grp = self.nxs.new_class(self.entry_grp, "measurement", "NXdata")
+        here = os.path.dirname(os.path.abspath(self.nxs.filename))
+        there = os.path.abspath(dataset.file.filename)
+        name = "images_%04i" % len(self.stored_input)
+        measurement_grp[name] = h5py.ExternalLink(os.path.relpath(there, here), dataset.name)
+        if "signal" not in measurement_grp.attrs:
+            measurement_grp.attrs["signal"] = name
+
     def process_one_frame(self, frame):
         """
         :param frame: 2d numpy array with an image to process
@@ -478,24 +533,25 @@ If the number of files is too large, use double quotes like "*.edf" """
         pos = self.get_pos(None, self._idx)
         shape = self.dataset.shape
         if pos.rot + 1 > shape[0]:
-            self.dataset.resize((pos.rot + 1, shape[1], shape[2]))
+            self.dataset.resize((pos.rot + 1,)+ shape[1:])
+            if self.dataset_error is not None:
+                self.dataset_error.resize((pos.rot + 1,)+ shape[1:])
         elif pos.index < 0 or pos.rot < 0 or pos.trans < 0:
             return
 
         res = self.worker.process(frame)
-#        _tth, I = self.ai.integrate1d(frame, self.npt_rad, safe=False,
-#                                      method=self.method, unit=self.unit)
-        self.dataset[pos.rot, pos.trans, :] = res.intensity
+        self.dataset[pos.rot, pos.trans, ...] = res.intensity
+        if res.sigma is not None:
+            self.dataset_error[pos.rot, pos.trans, ...] = res.sigma    
 
     def process(self):
         if self.dataset is None:
             self.makeHDF5()
         self.init_ai()
-        t0 = time.time()
-        # self._idx = -1
+        t0 = time.perf_counter()
         for f in self.inputfiles:
             self.process_one_file(f)
-        tot = time.time() - t0
+        tot = time.perf_counter() - t0
         cnt = self._idx + 1
         print(("Execution time for %i frames: %.3fs;"
                " Average execution time: %.1fms") %
@@ -521,24 +577,28 @@ If the number of files is too large, use double quotes like "*.edf" """
             return None
         else:
             return self.worker.ai
+
     @ai.setter
     def ai(self, value):
         if self.worker is None:
             self.worker = Worker(value, unit=self.unit, shapeOut=(1, self.npt_rad))
         else:
             self.worker.ai = value
+
     @property
     def method(self):
         if self.worker is not None:
             return self.worker.method
         return None
+
     @method.setter
     def method(self, value):
-        self.worker.set_method(value) 
-    
+        self.worker.set_method(value)
+
     @property
     def unit(self):
         return self.worker.unit
+
     @unit.setter
     def unit(self, value):
-        self.worker.unit = value
\ No newline at end of file
+        self.worker.unit = value
