@@ -4,7 +4,7 @@
 #    Project: Fast Azimuthal integration
 #             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2017-2019 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2017-2021 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -30,17 +30,14 @@
 translation table
 """
 
-from __future__ import absolute_import, print_function, with_statement, division
-
 __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "22/08/2018"
+__date__ = "04/10/2021"
 __status__ = "development"
 __docformat__ = 'restructuredtext'
 
-
 import os
 import logging
 import json
@@ -78,6 +75,7 @@ class BaseTransformation(object):
 
     This class relies on a user provided function which does the work.
     """
+
     def __init__(self, funct, param_names, pos_names=None):
         """Constructor of the class
 
@@ -96,7 +94,7 @@ class BaseTransformation(object):
             if key in self.variables:
                 raise RuntimeError("The keyword %s is already defined, please chose another variable name")
             self.variables[key] = numpy.NaN
-        self.codes = []
+        self.codes = {}
 
     def __call__(self, param, pos):
         """This makes the class instance behave like a function,
@@ -136,6 +134,7 @@ class GeometryTransformation(object):
     (dist, poni1, poni2, rot1, rot2, rot3)
     This function uses numexpr for formula evaluation.
     """
+
     def __init__(self, dist_expr, poni1_expr, poni2_expr,
                  rot1_expr, rot2_expr, rot3_expr,
                  param_names, pos_names=None, constants=None,
@@ -159,12 +158,19 @@ class GeometryTransformation(object):
             assert content in (self.__class__.__name__, "GeometryTransformation")
         if numexpr is None:
             raise RuntimeError("Geometry translation requires the *numexpr* package")
-        self.dist_expr = dist_expr
-        self.poni1_expr = poni1_expr
-        self.poni2_expr = poni2_expr
-        self.rot1_expr = rot1_expr
-        self.rot2_expr = rot2_expr
-        self.rot3_expr = rot3_expr
+        self.expressions = OrderedDict()
+        if dist_expr is not None:
+            self.expressions["dist"] = dist_expr
+        if poni1_expr is not None:
+            self.expressions["poni1"] = poni1_expr
+        if poni2_expr is not None:
+            self.expressions["poni2"] = poni2_expr
+        if rot1_expr is not None:
+            self.expressions["rot1"] = rot1_expr
+        if rot2_expr is not None:
+            self.expressions["rot2"] = rot2_expr
+        if rot3_expr is not None:
+            self.expressions["rot3"] = rot3_expr
 
         self.variables = {"pi": numpy.pi}
         if constants is not None:
@@ -177,11 +183,33 @@ class GeometryTransformation(object):
             self.pos_names = ("pos",)
         for key in self.param_names + self.pos_names:
             if key in self.variables:
-                raise RuntimeError("The keyword %s is already defined, please chose another variable name")
+                raise RuntimeError(f"The keyword `{key}` is already defined, please chose another variable name")
             self.variables[key] = numpy.NaN
+        self.codes = OrderedDict(((name, numexpr.NumExpr(expr)) for name, expr in self.expressions.items()))
 
-        self.codes = [numexpr.NumExpr(expr) for expr in (self.dist_expr, self.poni1_expr, self.poni2_expr,
-                                                         self.rot1_expr, self.rot2_expr, self.rot3_expr)]
+    @property
+    def dist_expr(self):
+        return self.expressions.get("dist")
+
+    @property
+    def poni1_expr(self):
+        return self.expressions.get("poni1")
+
+    @property
+    def poni2_expr(self):
+        return self.expressions.get("poni2")
+
+    @property
+    def rot1_expr(self):
+        return self.expressions.get("rot1")
+
+    @property
+    def rot2_expr(self):
+        return self.expressions.get("rot2")
+
+    @property
+    def rot3_expr(self):
+        return self.expressions.get("rot3")
 
     def __call__(self, param, pos):
         """This makes the class instance behave like a function,
@@ -193,7 +221,7 @@ class GeometryTransformation(object):
         :return: 6-tuple with (dist, poni1, poni2, rot1, rot2, rot3) as needed
             for pyFAI.
         """
-        res = []
+        res = {}
         variables = self.variables.copy()
         for name, value in zip(self.param_names, param):
             variables[name] = value
@@ -202,11 +230,11 @@ class GeometryTransformation(object):
         else:
             for name, value in zip(self.pos_names, pos):
                 variables[name] = value
-        for code in self.codes:
+        for name, code in self.codes.items():
             signa = [variables.get(name, numpy.NaN) for name in code.input_names]
-            res.append(float(code(*signa)))
+            res[name] = (float(code(*signa)))
             # could ne done in a single liner but harder to understand !
-        return PoniParam(*res)
+        return PoniParam(**res)
 
     def __repr__(self):
         res = ["GeometryTransformation with param: %s and pos: %s" % (self.param_names, self.pos_names),
@@ -248,6 +276,7 @@ class ExtendedTransformation(object):
 
     This function uses numexpr for formula evaluation.
     """
+
     def __init__(self, dist_expr=None, poni1_expr=None, poni2_expr=None,
                  rot1_expr=None, rot2_expr=None, rot3_expr=None, wavelength_expr=None,
                  param_names=None, pos_names=None, constants=None,
@@ -389,7 +418,7 @@ class Goniometer(object):
         self.param = param
         self.trans_function = trans_function
         self.detector = detector_factory(detector)
-        self.wavelength = wavelength
+        self._wavelength = wavelength
         if param_names is None and "param_names" in dir(trans_function):
             param_names = trans_function.param_names
         if param_names is not None:
@@ -405,29 +434,60 @@ class Goniometer(object):
     def __repr__(self):
         return "Goniometer with param %s    %s with %s" % (self.nt_param(*self.param), os.linesep, self.detector)
 
+    def get_wavelength(self):
+        wl_fct = self.trans_function.codes.get("wavelength")
+        if wl_fct is not None:
+            # check that wavelengt does not depend on the motor position
+            params = wl_fct.input_names
+            for motor in self.trans_function.pos_names:
+                if motor in params:
+                    logger.warning("Wavelength depends on motors, returning the default value")
+                    return self._wavelength
+            dummy_position = [0] * len(self.nt_pos._fields)
+            return self.trans_function(self.param, dummy_position).wavelength
+        else:
+            return self._wavelength
+
+    def set_wavelength(self, value):
+        if "wavelength" in self.trans_function.codes:
+            logger.warning("Wavelength is a fitted parameter, cannot be set. Please set fitted parameter")
+        else:
+            self._wavelength = value
+            
+    wavelength = property(get_wavelength, set_wavelength)
+
     def get_ai(self, position):
         """Creates an azimuthal integrator from the motor position
 
-        :param position: the goniometer position, a float for a 1 axis
-            goniometer
+        :param position: the goniometer position, a float for a 1 axis goniometer
         :return: A freshly build AzimuthalIntegrator
         """
         res = self.trans_function(self.param, position)
         params = {"detector": self.detector,
-                  "wavelength": self.wavelength}
+                  "wavelength": self._wavelength}
         for name, value in zip(res._fields, res):
             params[name] = value
         return AzimuthalIntegrator(**params)
 
-    def get_mg(self, positions):
+    def get_mg(self, positions, 
+               unit="2th_deg",
+               radial_range=(0, 180), azimuth_range=(-180, 180),
+               empty=0.0, chi_disc=180):
         """Creates a MultiGeometry integrator from a list of goniometer
         positions.
 
         :param positions: A list of goniometer positions
+        :param radial_range: common range for integration
+        :param azimuthal_range: common range for integration
+        :param empty: value for empty pixels
+        :param chi_disc: if 0, set the chi_discontinuity at 0, else pi
+
         :return: A freshly build multi-geometry
         """
         ais = [self.get_ai(pos) for pos in positions]
-        mg = MultiGeometry(ais)
+        mg = MultiGeometry(ais, unit=unit, 
+                           radial_range=radial_range, azimuth_range=azimuth_range, 
+                           empty=empty, chi_disc=chi_disc)
         return mg
 
     def to_dict(self):
@@ -464,6 +524,7 @@ class Goniometer(object):
                 f.write(json.dumps(dico, indent=2))
         except IOError:
             logger.error("IOError while writing to file %s", filename)
+
     write = save
 
     @classmethod
@@ -506,10 +567,10 @@ class Goniometer(object):
             # May be adapted for other classes of GeometryTransformation functions
             if content in ("GeometryTranslation", "GeometryTransformation"):
                 funct = GeometryTransformation(**tansfun)
-            elif content == "ExtendedTranformation":
+            elif content == "ExtendedTransformation":
                 funct = ExtendedTransformation(**tansfun)
             else:
-                raise RuntimeError("content= %s, not in in (GeometryTranslation, GeometryTransformation, ExtendedTranformation)")
+                raise RuntimeError(f"content={content}, not in in (GeometryTranslation, GeometryTransformation, ExtendedTransformation)")
         else:  # assume GeometryTransformation
             funct = GeometryTransformation(**tansfun)
 
@@ -524,6 +585,7 @@ class SingleGeometry(object):
     """This class represents a single geometry of a detector position on a
     goniometer arm
     """
+
     def __init__(self, label, image=None, metadata=None, pos_function=None,
                  control_points=None, calibrant=None, detector=None, geometry=None):
         """Constructor of the SingleGeometry class, used for calibrating a
@@ -658,15 +720,25 @@ class SingleGeometry(object):
 
         :return: Azimuthal Integrator instance
         """
-        geo = self.geometry_refinement.getPyFAI()
-        geo["detector"] = self.detector
-        return AzimuthalIntegrator(**geo)
+        config = self.geometry_refinement.get_config()
+        ai = AzimuthalIntegrator()
+        ai.set_config(config)
+        return ai
+    
+    def get_wavelength(self):
+        assert self.calibrant.wavelength == self.geometry_refinement.wavelength
+        return self.geometry_refinement.wavelength
+    def set_wavelength(self, value):
+        self.calibrant.setWavelength_change2th(value) 
+        self.geometry_refinement.set_wavelength(value)
+    wavelength = property(get_wavelength, set_wavelength)
 
 
 class GoniometerRefinement(Goniometer):
     """This class allow the translation of a goniometer geometry into a pyFAI
     geometry using a set of parameter to refine.
     """
+
     def __init__(self, param, pos_function, trans_function,
                  detector="Detector", wavelength=None, param_names=None, pos_names=None,
                  bounds=None):
@@ -746,6 +818,47 @@ class GoniometerRefinement(Goniometer):
                 npt += single.geometry_refinement.data.shape[0]
         return sumsquare / max(npt, 1)
 
+    def calc_param3(self, fit_param, free, const):
+        """Function that calculate the param vector 
+
+        :param fit_param: numpy array of float
+        :param free: names of the free parameters, array of same size as fit_param
+        :param const: dict with constant (non-fitted) parameters
+        :return: the parameter vector as in self.param 
+        """
+        param = [ ]
+        for name in self.nt_param._fields:
+            if name in free:
+                value = fit_param[free.index(name)]
+                param.append(value)
+            else:
+                param.append(const[name])
+        return param
+
+    def residu3(self, fit_param, free, const):
+        """Evaluate the cost function:
+        
+        :param fit_param: numpy array of float
+        :param free: names of the free parameters, array of same size as fit_param
+        :param const: dict with constant (non-fitted) parameters
+        :return: cost function value
+        """
+        sumsquare = 0.0
+        npt = 0
+        
+        param = self.calc_param3(fit_param, free, const)
+        
+        for single in self.single_geometries.values():
+            motor_pos = single.get_position()
+            single_param = self.trans_function(param, motor_pos)._asdict()
+            pyFAI_param = [single_param.get(name, 0.0)
+                           for name in ["dist", "poni1", "poni2", "rot1", "rot2", "rot3"]]
+            pyFAI_param.append(single_param.get("wavelength", self.wavelength) * 1e10)
+            if (single.geometry_refinement is not None) and (len(single.geometry_refinement.data) >= 1):
+                sumsquare += single.geometry_refinement.chi2_wavelength(pyFAI_param)
+                npt += single.geometry_refinement.data.shape[0]
+        return sumsquare / max(npt, 1)
+
     def chi2(self, param=None):
         """Calculate the average of the square of the error for a given parameter set
         """
@@ -758,13 +871,17 @@ class GoniometerRefinement(Goniometer):
         """Geometry refinement tool
 
         See https://docs.scipy.org/doc/scipy-0.18.1/reference/generated/scipy.optimize.minimize.html
+        
+        Nota: When upper and lower bounds are equal, the jacobian gets NaN since scipy 1.5.
 
         :param method: name of the minimizer
         :param options: options for the minimizer
+        :return: refined set of parameter
         """
         if method.lower() in ["simplex", "nelder-mead"]:
             method = "Nelder-Mead"
             bounds = None
+            logger.warning(f"No bounds for optimization method Nelder-Mead")
         else:
             bounds = self.bounds
         former_error = self.chi2()
@@ -801,6 +918,70 @@ class GoniometerRefinement(Goniometer):
                 sg.calibrant.setWavelength_change2th(former_wavelength)
             print(self.nt_param(*self.param))
         return self.param
+    
+    
+    
+    def refine3(self, fix=None, method="slsqp", verbose=True, **options):
+        """Geometry refinement tool
+        
+        :param fixed: list of parameters to be fixed (others are left free for refinement)
+        :param method: name of the minimizer
+        :param options: options for the minimizer
+        :return: refined set of parameter
+        """
+        if method.lower() in ["simplex", "nelder-mead"]:
+            method = "Nelder-Mead"
+            local_bounds = [(None, None) for i in self.param]
+            logger.warning(f"No bounds for optimization method Nelder-Mead")
+        else:
+            if self.bounds:
+                local_bounds = self.bounds
+            else:
+                local_bounds = [(None, None) for i in self.param]
+
+        fix = [] if fix is None else fix
+        free = []
+        param = []
+        bounds = []
+        const = {}
+        for i, name in enumerate(self.nt_param._fields):
+            value = self.param[i]
+            if name in fix:
+                const[name] = value
+            else:
+                minmax = local_bounds[i]
+                if (minmax[0] == minmax[1]) and (minmax[0] is not None):
+                    const[name] = value
+                else:
+                    free.append(name)
+                    param.append(value)
+                    bounds.append(minmax)
+        param = numpy.array(param)
+        if verbose:
+            print(f"Free parameters: {free}\nFixed: {const}")
+        old_delta_theta2 = self.residu3(param, free, const)
+
+        res = minimize(self.residu3, param, method=method,
+                       args=(free, const),
+                       bounds=bounds, tol=1e-12,    
+                       options=options)
+
+        new_delta_theta2 = self.residu3(res.x, free, const)
+
+        if verbose:
+            print(res)
+            print(f"Constrained Least square {old_delta_theta2} --> {new_delta_theta2}")
+
+        if new_delta_theta2 < old_delta_theta2:
+            i = abs(param - res.x).argmax()
+            if verbose:
+                print(f"maxdelta on {free[i]}: {param[i]} --> {res.x[i]} ")
+            
+            self.param = self.calc_param3(res.x, free, const)
+            return new_delta_theta2
+        else:
+            return old_delta_theta2
+        
 
     def set_bounds(self, name, mini=None, maxi=None):
         """Redefines the bounds for the refinement
@@ -824,7 +1005,6 @@ class GoniometerRefinement(Goniometer):
                     goniometer position
         :return: Goniometer object
         """
-
         with open(filename) as f:
             dico = json.load(f)
         assert dico["content"] == cls.file_version, "JSON file contains a goniometer calibration"
@@ -836,10 +1016,10 @@ class GoniometerRefinement(Goniometer):
             # May be adapted for other classes of GeometryTransformation functions
             if content in ("GeometryTranslation", "GeometryTransformation"):
                 funct = GeometryTransformation(**tansfun)
-            elif content == "ExtendedTranformation":
+            elif content == "ExtendedTransformation":
                 funct = ExtendedTransformation(**tansfun)
             else:
-                raise RuntimeError("content= %s, not in in (GeometryTranslation, GeometryTransformation, ExtendedTranformation)")
+                raise RuntimeError(f"content= {content}, not in in (GeometryTranslation, GeometryTransformation, ExtendedTranformation)")
         else:  # assume GeometryTransformation
             funct = GeometryTransformation(**tansfun)
 
@@ -849,3 +1029,13 @@ class GoniometerRefinement(Goniometer):
                     detector=detector,
                     wavelength=dico.get("wavelength"))
         return gonio
+
+    def get_wavelength(self):
+        return Goniometer.get_wavelength(self)
+
+    def set_wavelength(self, value):
+        Goniometer.set_wavelength(self, value)
+        for sg in self.single_geometries.values():
+            sg.set_wavelength(value)
+            
+    wavelength = property(get_wavelength, set_wavelength)
\ No newline at end of file
