@@ -23,20 +23,17 @@
 #
 # ###########################################################################*/
 
-from __future__ import absolute_import
-
 __authors__ = ["V. Valls"]
 __license__ = "MIT"
-__date__ = "01/02/2019"
+__date__ = "17/12/2021"
 
 import logging
 
 from silx.gui import qt
-from silx.utils import html
+import html
 
 import pyFAI.detectors
 
-
 _logger = logging.getLogger(__name__)
 
 
