@@ -1,10 +1,9 @@
 # -*- coding: utf-8 -*-
 #
 #    Project: Azimuthal integration
-#             https://github.com/silx-kit
+#             https://github.com/silx-kit/pyFAI
 #
-#
-#    Copyright (C) 2014-2019 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2014-2022 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #                            Giannis Ashiotis
@@ -29,8 +28,8 @@
 
 __authors__ = ["Jérôme Kieffer", "Giannis Ashiotis"]
 __license__ = "MIT"
-__date__ = "20/12/2019"
-__copyright__ = "2014-2019, ESRF, Grenoble"
+__date__ = "01/02/2022"
+__copyright__ = "2014-2021, ESRF, Grenoble"
 __contact__ = "jerome.kieffer@esrf.fr"
 
 import logging
@@ -42,10 +41,9 @@ if pyopencl:
     mf = pyopencl.mem_flags
 else:
     raise ImportError("pyopencl is not installed")
-from ..containers import Integrate1dtpl
+from ..containers import Integrate1dtpl, Integrate2dtpl
 from . import processing
 from . import get_x87_volatile_option
-from . import kernel_workgroup_size
 EventDescription = processing.EventDescription
 OpenclProcessing = processing.OpenclProcessing
 BufferDescription = processing.BufferDescription
@@ -59,9 +57,10 @@ class OCL_CSR_Integrator(OpenclProcessing):
 
     It also performs the preprocessing using the preproc kernel
     """
-    BLOCK_SIZE = 64
-    buffers = [BufferDescription("output", 1, numpy.float32, mf.WRITE_ONLY),
-               BufferDescription("output4", 4, numpy.float32, mf.WRITE_ONLY),
+    BLOCK_SIZE = 32
+    # Intel CPU driver calims preferred workgroup is 128 !
+    buffers = [BufferDescription("output", 1, numpy.float32, mf.READ_WRITE),
+               BufferDescription("output4", 4, numpy.float32, mf.READ_WRITE),
                BufferDescription("image_raw", 1, numpy.float32, mf.READ_ONLY),
                BufferDescription("image", 1, numpy.float32, mf.READ_WRITE),
                BufferDescription("variance", 1, numpy.float32, mf.READ_WRITE),
@@ -73,7 +72,7 @@ class OCL_CSR_Integrator(OpenclProcessing):
                BufferDescription("absorption", 1, numpy.float32, mf.READ_ONLY),
                BufferDescription("mask", 1, numpy.int8, mf.READ_ONLY),
                ]
-    kernel_files = ["pyfai:openCL/kahan.cl",
+    kernel_files = ["silx:opencl/doubleword.cl",
                     "pyfai:openCL/preprocess.cl",
                     "pyfai:openCL/memset.cl",
                     "pyfai:openCL/ocl_azim_CSR.cl"
@@ -87,9 +86,9 @@ class OCL_CSR_Integrator(OpenclProcessing):
                }
 
     def __init__(self, lut, image_size, checksum=None,
-                 empty=None, unit=None, bin_centers=None,
+                 empty=None, unit=None, bin_centers=None, azim_centers=None, mask_checksum=None,
                  ctx=None, devicetype="all", platformid=None, deviceid=None,
-                 block_size=None, profile=False):
+                 block_size=None, profile=False, extra_buffers=None):
         """
         :param lut: 3-tuple of arrays
             data: coefficient of the matrix in a 1D vector of float32 - size of nnz
@@ -100,6 +99,8 @@ class OCL_CSR_Integrator(OpenclProcessing):
         :param empty: value to be assigned to bins without contribution from any pixel
         :param unit: Storage for the unit related to the LUT
         :param bin_centers: the radial position of the bin_center, place_holder
+        :param azim_centers: the radial position of the bin_center, place_holder
+        :param mask_checksum: placeholder for the checksum of the mask
         :param ctx: actual working context, left to None for automatic
                     initialization from device type or platformid/deviceid
         :param devicetype: type of device, can be "CPU", "GPU", "ACC" or "ALL"
@@ -108,6 +109,7 @@ class OCL_CSR_Integrator(OpenclProcessing):
         :param block_size: preferred workgroup size, may vary depending on the outpcome of the compilation
         :param profile: switch on profiling to be able to profile at the kernel level,
                         store profiling elements (makes code slightly slower)
+        :param extra_buffers: List of additional buffer description  needed by derived classes
         """
         OpenclProcessing.__init__(self, ctx=ctx, devicetype=devicetype,
                                   platformid=platformid, deviceid=deviceid,
@@ -120,11 +122,13 @@ class OCL_CSR_Integrator(OpenclProcessing):
             raise RuntimeError("data.shape[0] != indices.shape[0]")
         self.data_size = self._data.shape[0]
         self.size = image_size
-        self.empty = empty or 0.0
+        self.empty = empty or 0
         self.unit = unit
         self.bin_centers = bin_centers
+        self.azim_centers = azim_centers
         # a few place-folders
-        self.pos0Range = self.pos1Range = self.check_mask = None
+        self.mask_checksum = mask_checksum
+        self.pos0_range = self.pos1_range = None
 
         if not checksum:
             checksum = calc_checksum(self._data)
@@ -136,25 +140,24 @@ class OCL_CSR_Integrator(OpenclProcessing):
                           "absorption": None,
                           "dark_variance": None}
 
-        if block_size is None:
-            block_size = self.BLOCK_SIZE
-
+        block_size = self.guess_workgroup_size(block_size)
         self.BLOCK_SIZE = min(block_size, self.device.max_work_group_size)
         self.workgroup_size = {}
-        self.wdim_bins = (self.bins * self.BLOCK_SIZE),
-        self.wdim_data = (self.size + self.BLOCK_SIZE - 1) & ~(self.BLOCK_SIZE - 1),
 
         self.buffers = [BufferDescription(i.name, i.size * self.size, i.dtype, i.flags)
                         for i in self.__class__.buffers]
 
+        if extra_buffers is not None:
+            self.buffers += extra_buffers
+
         self.buffers += [BufferDescription("data", self.data_size, numpy.float32, mf.READ_ONLY),
                          BufferDescription("indices", self.data_size, numpy.int32, mf.READ_ONLY),
                          BufferDescription("indptr", (self.bins + 1), numpy.int32, mf.READ_ONLY),
                          BufferDescription("sum_data", self.bins, numpy.float32, mf.WRITE_ONLY),
                          BufferDescription("sum_count", self.bins, numpy.float32, mf.WRITE_ONLY),
-                         BufferDescription("averint", self.bins, numpy.float32, mf.WRITE_ONLY),
-                         BufferDescription("stderr", self.bins, numpy.float32, mf.WRITE_ONLY),
-                         BufferDescription("stderrmean", self.bins, numpy.float32, mf.WRITE_ONLY),
+                         BufferDescription("averint", self.bins, numpy.float32, mf.READ_WRITE),
+                         BufferDescription("stderr", self.bins, numpy.float32, mf.READ_WRITE),
+                         BufferDescription("stderrmean", self.bins, numpy.float32, mf.READ_WRITE),
                          BufferDescription("merged", self.bins, numpy.float32, mf.WRITE_ONLY),
                          BufferDescription("merged8", (self.bins, 8), numpy.float32, mf.WRITE_ONLY),
                          ]
@@ -165,9 +168,25 @@ class OCL_CSR_Integrator(OpenclProcessing):
             self.set_kernel_arguments()
         except (pyopencl.MemoryError, pyopencl.LogicError) as error:
             raise MemoryError(error)
-        self.send_buffer(self._data, "data")
+        if numpy.allclose(self._data, numpy.ones(self._data.shape)):
+            self.cl_mem["data"] = None
+            self.cl_kernel_args["csr_sigma_clip4"]["data"] = None
+            self.cl_kernel_args["csr_integrate"]["data"] = None
+            self.cl_kernel_args["csr_integrate4"]["data"] = None
+        else:
+            self.send_buffer(self._data, "data")
         self.send_buffer(self._indices, "indices")
         self.send_buffer(self._indptr, "indptr")
+        if "amd" in  self.ctx.devices[0].platform.name.lower():
+            self.workgroup_size["csr_integrate4_single"] = (1, 1)  # Very bad performances on AMD GPU for diverging threads!
+
+    @property
+    def checksum(self):
+        return self.on_device.get("data")
+
+    @property
+    def check_mask(self):
+        return self.mask_checksum is not None
 
     def __copy__(self):
         """Shallow copy of the object
@@ -202,6 +221,48 @@ class OCL_CSR_Integrator(OpenclProcessing):
         memo[id(self)] = new_obj
         return new_obj
 
+    def guess_workgroup_size(self, block_size=None):
+        """Determines the optimal workgroup size.
+        
+        For azimuthal integration, especially the 2D variant, the 
+        smallest possible is the size of a warp/wavefront.
+        
+        The method can be overwritten by derived classes to select larger workgoup 
+        
+        :param block_size: Input workgroup size (block is the cuda name)
+        :return: the optimal workgoup size as integer  
+        """
+        device = self.ctx.devices[0]
+        platform = device.platform.name.lower()
+        try:
+            devtype = pyopencl.device_type.to_string(device.type).upper()
+        except ValueError:
+            # pocl does not describe itself as a CPU !
+            devtype = "CPU"
+
+        if block_size is None:
+            if "nvidia" in  platform:
+                try:
+                    block_size = device.warp_size_nv
+                except:
+                    block_size = 32
+            elif "amd" in  platform:
+                try:
+                    block_size = device.wavefront_width_amd
+                except:
+                    block_size = 64
+            elif "intel" in  platform:
+                block_size = 128
+            elif "portable" in platform and "CPU" in devtype:
+                block_size = 8
+            else:
+                block_size = min(device.max_work_group_size, self.BLOCK_SIZE)
+            self.force_workgroup_size = False
+        else:
+            self.force_workgroup_size = True
+            block_size = int(block_size)
+        return block_size
+
     def compile_kernels(self, kernel_file=None):
         """
         Call the OpenCL compiler
@@ -211,19 +272,26 @@ class OCL_CSR_Integrator(OpenclProcessing):
         kernel_file = kernel_file or self.kernel_files[-1]
         kernels = self.kernel_files[:-1] + [kernel_file]
 
+        compile_options = "-D NBINS=%i  -D NIMAGE=%i -D WORKGROUP_SIZE=%i" % \
+                          (self.bins, self.size, self.BLOCK_SIZE)
         try:
             default_compiler_options = self.get_compiler_options(x87_volatile=True)
         except AttributeError:  # Silx version too old
             logger.warning("Please upgrade to silx v0.10+")
             default_compiler_options = get_x87_volatile_option(self.ctx)
-        compile_options = "-D NBINS=%i  -D NIMAGE=%i -D WORKGROUP_SIZE=%i" % \
-                          (self.bins, self.size, self.BLOCK_SIZE)
+
         if default_compiler_options:
             compile_options += " " + default_compiler_options
         OpenclProcessing.compile_kernels(self, kernels, compile_options)
-        for kernel_name, kernel in self.kernels.get_kernels().items():
-            wg = kernel_workgroup_size(self.program, kernel)
-            self.workgroup_size[kernel_name] = (min(wg, self.BLOCK_SIZE),)  # this is a tuple
+        for kernel_name in self.kernels.__dict__:
+            if kernel_name.startswith("_"):
+                continue
+            if self.force_workgroup_size:
+                self.workgroup_size[kernel_name] = (self.BLOCK_SIZE, self.BLOCK_SIZE)
+            else:
+                wg_max = min(self.BLOCK_SIZE, self.kernels.max_workgroup_size(kernel_name))
+                wg_min = min(self.BLOCK_SIZE, self.kernels.min_workgroup_size(kernel_name))
+                self.workgroup_size[kernel_name] = (wg_min, wg_max)
 
     def set_kernel_arguments(self):
         """Tie arguments of OpenCL kernel-functions to the actual kernels
@@ -258,7 +326,6 @@ class OCL_CSR_Integrator(OpenclProcessing):
                                                             ("sum_data", self.cl_mem["sum_data"]),
                                                             ("sum_count", self.cl_mem["sum_count"]),
                                                             ("merged", self.cl_mem["merged"])))
-
         self.cl_kernel_args["corrections4"] = OrderedDict((("image", self.cl_mem["image"]),
                                                            ("poissonian", numpy.int8(0)),
                                                            ("variance", self.cl_mem["variance"]),
@@ -286,6 +353,8 @@ class OCL_CSR_Integrator(OpenclProcessing):
                                                             ("data", self.cl_mem["data"]),
                                                             ("indices", self.cl_mem["indices"]),
                                                             ("indptr", self.cl_mem["indptr"]),
+                                                            ("empty", numpy.float32(self.empty)),
+                                                            ("azimuthal", numpy.int8(1)),
                                                             ("merged8", self.cl_mem["merged8"]),
                                                             ("averint", self.cl_mem["averint"]),
                                                             ("stderr", self.cl_mem["stderr"]),
@@ -297,7 +366,8 @@ class OCL_CSR_Integrator(OpenclProcessing):
                                                               ("indptr", self.cl_mem["indptr"]),
                                                               ("cutoff", numpy.float32(5)),
                                                               ("cycle", numpy.int32(5)),
-                                                              ("azimuthal", numpy.int32(0)),
+                                                              ("azimuthal", numpy.int8(1)),
+                                                              ("empty", numpy.float32(self.empty)),
                                                               ("merged8", self.cl_mem["merged8"]),
                                                               ("averint", self.cl_mem["averint"]),
                                                               ("stderr", self.cl_mem["stderr"]),
@@ -388,9 +458,9 @@ class OCL_CSR_Integrator(OpenclProcessing):
         events = []
         with self.sem:
             self.send_buffer(data, "image")
-            wg = self.workgroup_size["memset_out"]
-            wdim_bins = (self.bins + wg[0] - 1) & ~(wg[0] - 1),
-            memset = self.kernels.memset_out(self.queue, wdim_bins, wg, *list(self.cl_kernel_args["memset_out"].values()))
+            wg = max(self.workgroup_size["memset_out"])
+            wdim_bins = (self.bins + wg - 1) & ~(wg - 1),
+            memset = self.kernels.memset_out(self.queue, wdim_bins, (wg,), *list(self.cl_kernel_args["memset_out"].values()))
             events.append(EventDescription("memset_out", memset))
             kw_corr = self.cl_kernel_args["corrections"]
             kw_int = self.cl_kernel_args["csr_integrate"]
@@ -466,8 +536,9 @@ class OCL_CSR_Integrator(OpenclProcessing):
                 do_absorption = numpy.int8(0)
             kw_corr["do_absorption"] = do_absorption
 
-            wg = self.workgroup_size["corrections"]
-            ev = self.kernels.corrections(self.queue, self.wdim_data, wg, *list(kw_corr.values()))
+            wg = max(self.workgroup_size["corrections"])
+            wdim_data = (self.size + wg - 1) & ~(wg - 1),
+            ev = self.kernels.corrections(self.queue, wdim_data, (wg,), *list(kw_corr.values()))
             events.append(EventDescription("corrections", ev))
 
             if preprocess_only:
@@ -476,37 +547,48 @@ class OCL_CSR_Integrator(OpenclProcessing):
                 events.append(EventDescription("copy D->H image", ev))
                 if self.profile:
                     self.events += events
-                ev.wait()
+                # ev.wait()
                 return image
-            wg = self.workgroup_size["csr_integrate"][0]
 
-            wdim_bins = (self.bins * wg),
-            if wg == 1:
-                integrate = self.kernels.csr_integrate_single(self.queue, wdim_bins, (wg,), *kw_int.values())
-                events.append(EventDescription("integrate_single", integrate))
+            wg_min, wg_max = self.workgroup_size["csr_integrate"]
+            if wg_max == 1:
+                wg_min, wg_max = self.workgroup_size["csr_integrate_single"]
+                wdim_bins = (self.bins + wg_max - 1) & ~(wg_max - 1),
+                integrate = self.kernels.csr_integrate_single(self.queue, wdim_bins, (wg_max,), *kw_int.values())
+                events.append(EventDescription("csr_integrate_single", integrate))
             else:
-                integrate = self.kernels.csr_integrate(self.queue, wdim_bins, (wg,), *kw_int.values())
-                events.append(EventDescription("integrate", integrate))
+                wdim_bins = (self.bins * wg_min),
+                integrate = self.kernels.csr_integrate(self.queue, wdim_bins, (wg_min,), *kw_int.values())
+                events.append(EventDescription("csr_integrate", integrate))
             if out_merged is None:
                 merged = numpy.empty(self.bins, dtype=numpy.float32)
+            elif out_merged is False:
+                merged = None
             else:
                 merged = out_merged.data
             if out_sum_count is None:
                 sum_count = numpy.empty(self.bins, dtype=numpy.float32)
+            elif out_sum_count is False:
+                sum_count = None
             else:
                 sum_count = out_sum_count.data
             if out_sum_data is None:
                 sum_data = numpy.empty(self.bins, dtype=numpy.float32)
+            elif out_sum_data is False:
+                sum_data = None
             else:
                 sum_data = out_sum_data.data
 
-            ev = pyopencl.enqueue_copy(self.queue, merged, self.cl_mem["merged"])
-            events.append(EventDescription("copy D->H merged", ev))
-            ev = pyopencl.enqueue_copy(self.queue, sum_data, self.cl_mem["sum_data"])
-            events.append(EventDescription("copy D->H sum_data", ev))
-            ev = pyopencl.enqueue_copy(self.queue, sum_count, self.cl_mem["sum_count"])
-            events.append(EventDescription("copy D->H sum_count", ev))
-            ev.wait()
+            if merged is not None:
+                ev = pyopencl.enqueue_copy(self.queue, merged, self.cl_mem["merged"])
+                events.append(EventDescription("copy D->H merged", ev))
+            if  sum_data is not None:
+                ev = pyopencl.enqueue_copy(self.queue, sum_data, self.cl_mem["sum_data"])
+                events.append(EventDescription("copy D->H sum_data", ev))
+            if sum_count is not None:
+                ev = pyopencl.enqueue_copy(self.queue, sum_count, self.cl_mem["sum_count"])
+                events.append(EventDescription("copy D->H sum_count", ev))
+            # ev.wait()
         if self.profile:
             self.events += events
         return merged, sum_data, sum_count
@@ -537,7 +619,7 @@ class OCL_CSR_Integrator(OpenclProcessing):
         :param dark: array of same shape as data for pre-processing
         :param dummy: value for invalid data
         :param delta_dummy: precesion for dummy assessement
-        :param poissonian: set to use signal as variance (minimum 1)
+        :param poissonian: set to use signal as variance (minimum 1), set to False to use azimuthal model
         :param variance: array of same shape as data for pre-processing
         :param dark_variance: array of same shape as data for pre-processing
         :param flat: array of same shape as data for pre-processing
@@ -558,9 +640,9 @@ class OCL_CSR_Integrator(OpenclProcessing):
         events = []
         with self.sem:
             self.send_buffer(data, "image")
-            wg = self.workgroup_size["memset_ng"]
-            wdim_bins = (self.bins + wg[0] - 1) & ~(wg[0] - 1),
-            memset = self.kernels.memset_out(self.queue, wdim_bins, wg, *list(self.cl_kernel_args["memset_ng"].values()))
+            wg = max(self.workgroup_size["memset_ng"])
+            wdim_bins = (self.bins + wg - 1) & ~(wg - 1),
+            memset = self.kernels.memset_out(self.queue, wdim_bins, (wg,), *list(self.cl_kernel_args["memset_ng"].values()))
             events.append(EventDescription("memset_ng", memset))
             kw_corr = self.cl_kernel_args["corrections4"]
             kw_int = self.cl_kernel_args["csr_integrate4"]
@@ -583,6 +665,7 @@ class OCL_CSR_Integrator(OpenclProcessing):
             kw_corr["normalization_factor"] = numpy.float32(normalization_factor)
 
             kw_corr["poissonian"] = numpy.int8(1 if poissonian else 0)
+            kw_int["azimuthal"] = numpy.int8(poissonian is False)
             if variance is not None:
                 self.send_buffer(variance, "variance")
             if dark_variance is not None:
@@ -645,44 +728,70 @@ class OCL_CSR_Integrator(OpenclProcessing):
                 do_absorption = numpy.int8(0)
             kw_corr["do_absorption"] = do_absorption
 
-            wg = self.workgroup_size["corrections4"]
-            ev = self.kernels.corrections4(self.queue, self.wdim_data, wg, *list(kw_corr.values()))
-            events.append(EventDescription("corrections", ev))
+            wg = max(self.workgroup_size["corrections4"])
+            wdim_data = (self.size + wg - 1) & ~(wg - 1),
+            ev = self.kernels.corrections4(self.queue, wdim_data, (wg,), *list(kw_corr.values()))
+            events.append(EventDescription("corrections4", ev))
 
-            wg = self.workgroup_size["csr_integrate4"][0]
-
-            wdim_bins = (self.bins * wg),
-            if wg == 1:
+            kw_int["empty"] = dummy
+            wg_min, wg_max = self.workgroup_size["csr_integrate4"]
+            if  wg_max == 1:
+                wg = max(self.workgroup_size["csr_integrate4_single"])
+                wdim_bins = (self.bins + wg - 1) & ~(wg - 1),
                 integrate = self.kernels.csr_integrate4_single(self.queue, wdim_bins, (wg,), *kw_int.values())
-                events.append(EventDescription("integrate4_single", integrate))
+                events.append(EventDescription("csr_integrate4_single", integrate))
             else:
-                integrate = self.kernels.csr_integrate4(self.queue, wdim_bins, (wg,), *kw_int.values())
-                events.append(EventDescription("integrate4", integrate))
+                wdim_bins = (self.bins * wg_min),
+                integrate = self.kernels.csr_integrate4(self.queue, wdim_bins, (wg_min,), *kw_int.values())
+                events.append(EventDescription("csr_integrate4", integrate))
 
             if out_merged is None:
                 merged = numpy.empty((self.bins, 8), dtype=numpy.float32)
+            elif out_merged is False:
+                merged = None
             else:
                 merged = out_merged.data
             if out_avgint is None:
                 avgint = numpy.empty(self.bins, dtype=numpy.float32)
+            elif out_avgint is False:
+                avgint = None
             else:
                 avgint = out_avgint.data
             if out_stderr is None:
                 stderr = numpy.empty(self.bins, dtype=numpy.float32)
+            elif out_stderr is  False:
+                stderr = None
             else:
                 stderr = out_stderr.data
 
-            ev = pyopencl.enqueue_copy(self.queue, avgint, self.cl_mem["averint"])
-            events.append(EventDescription("copy D->H avgint", ev))
+            if avgint is not None:
+                ev = pyopencl.enqueue_copy(self.queue, avgint, self.cl_mem["averint"])
+                events.append(EventDescription("copy D->H avgint", ev))
+            if stderr is not None:
+                ev = pyopencl.enqueue_copy(self.queue, stderr, self.cl_mem["stderr"])
+                events.append(EventDescription("copy D->H stderr", ev))
+            if self.azim_centers is None:
+                if merged is None:
+                    res = Integrate1dtpl(self.bin_centers, avgint, stderr, None, None, None, None)
+                else:
+                    ev = pyopencl.enqueue_copy(self.queue, merged, self.cl_mem["merged8"])
+                    events.append(EventDescription("copy D->H merged8", ev))
+                    res = Integrate1dtpl(self.bin_centers, avgint, stderr, merged[:, 0], merged[:, 2], merged[:, 4], merged[:, 6])
+            else:  # 2D case
+                outshape = self.bin_centers.size, self.azim_centers.size
+                if merged is None:  # "radial azimuthal intensity sigma"
+                    res = Integrate2dtpl(self.bin_centers, self.azim_centers,
+                                         avgint.reshape(outshape).T, stderr.reshape(outshape).T,
+                                         None, None, None, None)
+                else:
+                    ev = pyopencl.enqueue_copy(self.queue, merged, self.cl_mem["merged8"])
+                    events.append(EventDescription("copy D->H merged8", ev))
+                    res = Integrate2dtpl(self.bin_centers, self.azim_centers,
+                                         avgint.reshape(outshape).T, stderr.reshape(outshape).T,
+                                         merged[:, 0].reshape(outshape).T, merged[:, 2].reshape(outshape).T, merged[:, 4].reshape(outshape).T, merged[:, 6].reshape(outshape).T)
 
-            ev = pyopencl.enqueue_copy(self.queue, stderr, self.cl_mem["stderr"])
-            events.append(EventDescription("copy D->H stderr", ev))
-            ev = pyopencl.enqueue_copy(self.queue, merged, self.cl_mem["merged8"])
-            events.append(EventDescription("copy D->H merged8", ev))
         if self.profile:
             self.events += events
-        res = Integrate1dtpl(self.bin_centers, avgint, stderr, merged[:, 0], merged[:, 2], merged[:, 4], merged[:, 6])
-        "position intensity error signal variance normalization count"
         return res
 
     def sigma_clip(self, data, dark=None, dummy=None, delta_dummy=None,
@@ -695,12 +804,12 @@ class OCL_CSR_Integrator(OpenclProcessing):
                    cutoff=4.0, cycle=5,
                    out_avgint=None, out_stderr=None, out_merged=None):
         """
-        Perform a sigma-clipping iterative filter within each along each row. 
+        Perform a sigma-clipping iterative filter within each along each row.
         see the doc of scipy.stats.sigmaclip for more descriptions.
-        
+
         If the error model is "azimuthal": the variance is the variance within a bin,
         which is refined at each iteration, can be costly !
-        
+
         Else, the error is propagated according to:
 
         .. math::
@@ -728,7 +837,7 @@ class OCL_CSR_Integrator(OpenclProcessing):
         :param safe: if True (default) compares arrays on GPU according to their checksum, unless, use the buffer location is used
         :param preprocess_only: return the dark subtracted; flat field & solidangle & polarization corrected image, else
         :param normalization_factor: divide raw signal by this value
-        :param cutoff: discard all points with |value - avg| > cutoff * sigma. 3-4 is quite common 
+        :param cutoff: discard all points with |value - avg| > cutoff * sigma. 3-4 is quite common
         :param cycle: perform at maximum this number of cycles. 5 is common.
         :param out_avgint: destination array or pyopencl array for sum of all data
         :param out_stderr: destination array or pyopencl array for sum of the number of pixels
@@ -742,9 +851,9 @@ class OCL_CSR_Integrator(OpenclProcessing):
             error_model = ""
         with self.sem:
             self.send_buffer(data, "image")
-            wg = self.workgroup_size["memset_ng"]
-            wdim_bins = (self.bins + wg[0] - 1) & ~(wg[0] - 1),
-            memset = self.kernels.memset_out(self.queue, wdim_bins, wg, *list(self.cl_kernel_args["memset_ng"].values()))
+            wg = max(self.workgroup_size["memset_ng"])
+            wdim_bins = (self.bins + wg - 1) & ~(wg - 1),
+            memset = self.kernels.memset_out(self.queue, wdim_bins, (wg,), *list(self.cl_kernel_args["memset_ng"].values()))
             events.append(EventDescription("memset_ng", memset))
             kw_corr = self.cl_kernel_args["corrections4"]
             kw_int = self.cl_kernel_args["csr_sigma_clip4"]
@@ -830,26 +939,20 @@ class OCL_CSR_Integrator(OpenclProcessing):
                 do_absorption = numpy.int8(0)
             kw_corr["do_absorption"] = do_absorption
 
-            wg = self.workgroup_size["corrections4"]
-            ev = self.kernels.corrections4(self.queue, self.wdim_data, wg, *list(kw_corr.values()))
+            wg = max(self.workgroup_size["corrections4"])
+            wdim_data = (self.size + wg - 1) & ~(wg - 1),
+            ev = self.kernels.corrections4(self.queue, wdim_data, (wg,), *list(kw_corr.values()))
             events.append(EventDescription("corrections", ev))
 
-            wg = self.workgroup_size["csr_sigma_clip4"][0]
             kw_int["cutoff"] = numpy.float32(cutoff)
             kw_int["cycle"] = numpy.int32(cycle)
-            if error_model.startswith("azim"):
-                kw_int["azimuthal"] = numpy.int32(1)
-            else:
-                kw_int["azimuthal"] = numpy.int32(0)
+            kw_int["azimuthal"] = numpy.int8(error_model.startswith("azim"))
 
-            wdim_bins = (self.bins * wg),
-            if wg == 1:
-                raise RuntimeError("csr_sigma_clip4 is not yet available in single threaded OpenCL !")
-                integrate = self.kernels.csr_integrate4_single(self.queue, wdim_bins, (wg,), *kw_int.values())
-                events.append(EventDescription("integrate4_single", integrate))
-            else:
-                integrate = self.kernels.csr_sigma_clip4(self.queue, wdim_bins, (wg,), *kw_int.values())
-                events.append(EventDescription("csr_sigma_clip4", integrate))
+            wg_min, wg_max = self.workgroup_size["csr_sigma_clip4"]
+
+            wdim_bins = (self.bins * wg_min),
+            integrate = self.kernels.csr_sigma_clip4(self.queue, wdim_bins, (wg_min,), *kw_int.values())
+            events.append(EventDescription("csr_sigma_clip4", integrate))
 
             if out_merged is None:
                 merged = numpy.empty((self.bins, 8), dtype=numpy.float32)
