@@ -1,9 +1,9 @@
-# -*- coding: utf-8 -*-
+# coding: utf-8
 #
-#    Project: silx
+#    Project: pyFAI
 #             https://github.com/silx-kit/silx
 #
-#    Copyright (C) 2012-2016  European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2012-2020  European Synchrotron Radiation Facility, Grenoble, France
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
 # in the Software without restriction, including without limitation the rights
@@ -24,7 +24,7 @@
 
 __authors__ = ["J. Kieffer"]
 __license__ = "MIT"
-__date__ = "12/06/2019"
+__date__ = "20/01/2021"
 
 import unittest
 from ...test.utilstest import UtilsTest
@@ -37,7 +37,15 @@ def suite():
         from . import test_addition
         from . import test_ocl_histo
         from . import test_ocl_azim_csr
+        from . import test_ocl_azim_lut
+        from . import test_peak_finder
+        from . import test_ocl_sort
+        from . import test_openCL
         testSuite.addTests(test_addition.suite())
+        testSuite.addTests(test_openCL.suite())
         testSuite.addTests(test_ocl_histo.suite())
         testSuite.addTests(test_ocl_azim_csr.suite())
+        testSuite.addTests(test_ocl_azim_lut.suite())
+        testSuite.addTests(test_peak_finder.suite())
+        testSuite.addTests(test_ocl_sort.suite())
     return testSuite
