@@ -4,7 +4,7 @@
 #    Project: Azimuthal integration
 #             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2015-2018 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2015-2021 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -28,13 +28,11 @@
 
 """test suite for Azimuthal integrator class"""
 
-from __future__ import absolute_import, division, print_function
-
 __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "19/11/2019"
+__date__ = "10/01/2022"
 
 import unittest
 import os
@@ -43,10 +41,8 @@ import logging
 import time
 import copy
 import fabio
-import tempfile
 import gc
 import shutil
-from . import utilstest
 from .utilstest import UtilsTest
 logger = logging.getLogger(__name__)
 from ..azimuthalIntegrator import AzimuthalIntegrator
@@ -55,8 +51,6 @@ if logger.getEffectiveLevel() <= logging.DEBUG:
     import pylab
 from pyFAI import units, detector_factory
 from ..utils import mathutil
-from ..third_party import six
-from pyFAI.utils.decorators import depreclog
 
 
 @unittest.skipIf(UtilsTest.low_mem, "test using >500M")
@@ -153,17 +147,19 @@ class TestAzimHalfFrelon(unittest.TestCase):
             self.__class__.ai.reset()
         except Exception as e:
             logger.error(e)
+        gc.collect()
 
     @unittest.skipIf(UtilsTest.low_mem, "test using >100Mb")
     def test_numpy_vs_fit2d(self):
         """
         Compare numpy histogram with results of fit2d
         """
-        tth, I = self.ai.integrate1d(self.data,
-                                     len(self.fit2d),
-                                     filename=self.tmpfiles["numpy"],
-                                     correctSolidAngle=False,
-                                     unit="2th_deg")
+        tth, I = self.ai.integrate1d_ng(self.data,
+                                        len(self.fit2d),
+                                        filename=self.tmpfiles["numpy"],
+                                        correctSolidAngle=False,
+                                        method=("no", "histogram", "cython"),
+                                        unit="2th_deg")
         rwp = mathutil.rwp((tth, I), self.fit2d.T)
         logger.info("Rwp numpy/fit2d = %.3f", rwp)
         if logger.getEffectiveLevel() == logging.DEBUG:
@@ -176,7 +172,7 @@ class TestAzimHalfFrelon(unittest.TestCase):
             handles, labels = sp.get_legend_handles_labels()
             fig.legend(handles, labels)
             fig.show()
-            six.moves.input("Press enter to quit")
+            input("Press enter to quit")
         self.assertLess(rwp, 11, "Rwp numpy/fit2d: %.3f" % rwp)
 
     @unittest.skipIf(UtilsTest.low_mem, "test using >100Mb")
@@ -184,14 +180,12 @@ class TestAzimHalfFrelon(unittest.TestCase):
         """
         Compare cython histogram with results of fit2d
         """
-        tth, I = self.ai.integrate1d(self.data,
+        tth, I = self.ai.integrate1d_ng(self.data,
                                      len(self.fit2d),
                                      filename=self.tmpfiles["cython"],
                                      correctSolidAngle=False,
                                      unit='2th_deg',
-                                     method="cython")
-        # logger.info(tth)
-        # logger.info(I)
+                                     method=("no", "histogram", "cython"))
         rwp = mathutil.rwp((tth, I), self.fit2d.T)
         logger.info("Rwp cython/fit2d = %.3f", rwp)
         if logger.getEffectiveLevel() == logging.DEBUG:
@@ -204,7 +198,7 @@ class TestAzimHalfFrelon(unittest.TestCase):
             handles, labels = sp.get_legend_handles_labels()
             fig.legend(handles, labels)
             fig.show()
-            six.moves.input("Press enter to quit")
+            input("Press enter to quit")
         self.assertLess(rwp, 11, "Rwp cython/fit2d: %.3f" % rwp)
 
     @unittest.skipIf(UtilsTest.low_mem, "test using >200M")
@@ -215,17 +209,17 @@ class TestAzimHalfFrelon(unittest.TestCase):
         logger.info(self.ai.__repr__())
         self.ai.corner_array(self.data.shape, unit=units.TTH_RAD, scale=False)
         # this was just to enforce the initalization of the array
-        t0 = time.time()
+        t0 = time.perf_counter()
         logger.info("in test_cythonSP_vs_fit2d Before SP")
 
-        tth, I = self.ai.integrate1d(self.data,
-                                     len(self.fit2d),
-                                     filename=self.tmpfiles["cythonSP"],
-                                     method="splitpixel",
-                                     correctSolidAngle=False,
-                                     unit="2th_deg")
+        tth, I = self.ai.integrate1d_ng(self.data,
+                                        len(self.fit2d),
+                                        filename=self.tmpfiles["cythonSP"],
+                                        method=("full", "histogram", "cython"),
+                                        correctSolidAngle=False,
+                                        unit="2th_deg")
         logger.info("in test_cythonSP_vs_fit2d Before")
-        t1 = time.time() - t0
+        t1 = time.perf_counter() - t0
         rwp = mathutil.rwp((tth, I), self.fit2d.T)
         logger.info("Rwp cythonSP(t=%.3fs)/fit2d = %.3f", t1, rwp)
         if logger.getEffectiveLevel() == logging.DEBUG:
@@ -238,7 +232,7 @@ class TestAzimHalfFrelon(unittest.TestCase):
             handles, labels = sp.get_legend_handles_labels()
             fig.legend(handles, labels)
             fig.show()
-            six.moves.input("Press enter to quit")
+            input("Press enter to quit")
         self.assertLess(rwp, 11, "Rwp cythonSP/fit2d: %.3f" % rwp)
 
     @unittest.skipIf(UtilsTest.low_mem, "test using >100Mb")
@@ -246,30 +240,28 @@ class TestAzimHalfFrelon(unittest.TestCase):
         """
         Compare cython histogram with numpy histogram
         """
-        tth_np, I_np = self.ai.integrate1d(self.__class__.data,
-                                           len(self.fit2d),
-                                           correctSolidAngle=False,
-                                           unit="2th_deg",
-                                           method="numpy")
-        tth_cy, I_cy = self.ai.integrate1d(self.__class__.data,
-                                           len(self.fit2d),
-                                           correctSolidAngle=False,
-                                           unit="2th_deg",
-                                           method="cython")
-        logger.info("before xrpd_splitPixel")
-        tth_sp, I_sp = self.ai.integrate1d(self.__class__.data,
-                                           len(self.fit2d),
-                                           correctSolidAngle=False,
-                                           unit="2th_deg",
-                                           method="splitpixel")
-        logger.info("After xrpd_splitPixel")
+        tth_np, I_np = self.ai.integrate1d_ng(self.__class__.data,
+                                              len(self.fit2d),
+                                              correctSolidAngle=False,
+                                              unit="2th_deg",
+                                              method=("no", "histogram", "numpy"))
+        tth_cy, I_cy = self.ai.integrate1d_ng(self.__class__.data,
+                                              len(self.fit2d),
+                                              correctSolidAngle=False,
+                                              unit="2th_deg",
+                                              method=("no", "histogram", "cython"))
+        tth_sp, I_sp = self.ai.integrate1d_ng(self.__class__.data,
+                                              len(self.fit2d),
+                                              correctSolidAngle=False,
+                                              unit="2th_deg",
+                                              method=("full", "histogram", "cython"))
+
         rwp = mathutil.rwp((tth_cy, I_cy), (tth_np, I_np))
-        logger.info("Rwp = %.3f", rwp)
+        logger.info("Histogram Cython/Numpy Rwp = %.3f", rwp)
         if logger.getEffectiveLevel() == logging.DEBUG:
             logging.info("Plotting results")
-            fig = pylab.figure()
+            fig, sp = pylab.subplots()
             fig.suptitle('Numpy Histogram vs Cython: Rwp=%.3f' % rwp)
-            sp = fig.add_subplot(111)
             sp.plot(self.fit2d.T[0], self.fit2d.T[1], "-y", label='fit2d')
             sp.plot(tth_np, I_np, "-b", label='numpy')
             sp.plot(tth_cy, I_cy, "-r", label="cython")
@@ -277,7 +269,7 @@ class TestAzimHalfFrelon(unittest.TestCase):
             handles, labels = sp.get_legend_handles_labels()
             fig.legend(handles, labels)
             fig.show()
-            six.moves.input("Press enter to quit")
+            input("Press enter to quit")
 
         self.assertLess(rwp, 3, "Rwp cython/numpy: %.3f" % rwp)
 
@@ -299,18 +291,19 @@ class TestAzimHalfFrelon(unittest.TestCase):
         ocl = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="bbox_ocl_csr")
         rwp = mathutil.rwp(ref, ocl)
         logger.info("test_medfilt1d median Rwp = %.3f", rwp)
-        self.assertLess(rwp, 1, "Rwp medfilt1d Numpy/OpenCL: %.3f" % rwp)
+#        print("cython", ref.intensity.min(), ref.intensity.max(), ref.intensity.mean(), ref.intensity.std())
+#        print("opencl", ocl.intensity.min(), ocl.intensity.max(), ocl.intensity.mean(), ocl.intensity.std())
+        self.assertLess(rwp, 1, "Rwp medfilt1d Cython/OpenCL: %.3f" % rwp)
 
         ref = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="bbox_csr", percentile=(20, 80))
         ocl = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="bbox_ocl_csr", percentile=(20, 80))
         rwp = mathutil.rwp(ref, ocl)
         logger.info("test_medfilt1d trimmed-mean Rwp = %.3f", rwp)
-        self.assertLess(rwp, 3, "Rwp trimmed-mean Numpy/OpenCL: %.3f" % rwp)
+        self.assertLess(rwp, 3, "Rwp trimmed-mean Cython/OpenCL: %.3f" % rwp)
         ref = ocl = rwp = None
-        gc.collect()
 
     def test_radial(self):
-
+        "Non regression for #1602"
         res = self.ai.integrate_radial(self.data, npt=360, npt_rad=10,
                                        radial_range=(3.6, 3.9), radial_unit="2th_deg")
         self.assertLess(res[0].min(), -179, "chi min at -180")
@@ -322,48 +315,59 @@ class TestAzimHalfFrelon(unittest.TestCase):
                                        radial_range=(3.6, 3.9), radial_unit="2th_deg", unit="chi_rad")
         self.assertLess(res[0].min(), -3, "chi min at -3rad")
         self.assertGreater(res[0].max(), 0, "chi max at +3rad")
+        self.assertGreater(res[1].min(), 120, "intensity min in ok")
+        self.assertLess(res[1].max(), 10000, "intensity max in ok")
+
+        res = self.ai.integrate_radial(self.data, npt=360, npt_rad=10,
+                                       radial_range=(3.6, 3.9), radial_unit="2th_deg",
+                                       method=("full", "CSR", "opencl"))
+        self.assertLess(res[0].min(), -179, "chi min at -180")
+        self.assertGreater(res[0].max(), 179, "chi max at +180")
+        self.assertGreater(res[1].min(), 120, "intensity min in ok")
+        self.assertLess(res[1].max(), 10000, "intensity max in ok")
 
 
 class TestFlatimage(unittest.TestCase):
     """test the caking of a flat image"""
-    epsilon = 1e-4
+
+    @classmethod
+    def setUpClass(cls):
+        cls.epsilon = 1e-4
+        cls.shape = (200, 201)
+        cls.data = numpy.ones(cls.shape, dtype="float64")
+        det = Detector(1e-4, 1e-4, max_shape=cls.shape)
+        cls.ai = AzimuthalIntegrator(0.1, 1e-2, 1e-2, detector=det)
+
+    @classmethod
+    def tearDownClass(cls):
+        cls.epsilon = cls.shape = cls.data = cls.ai = None
 
     def test_splitPixel(self):
-        shape = (200, 201)
-        data = numpy.ones(shape, dtype="float64")
-        det = Detector(1e-4, 1e-4, max_shape=shape)
-        ai = AzimuthalIntegrator(0.1, 1e-2, 1e-2, detector=det)
-
-        I = ai.integrate2d(data, 256, 2256, correctSolidAngle=False, dummy=-1.0,
-                           method='splitpixel', unit='2th_deg')[0]
+        res = self.ai.integrate2d(self.data, 256, 256, correctSolidAngle=False, dummy=-1.0,
+                           method='splitpixel', unit='2th_deg')
+        I = res[0]
         if logger.getEffectiveLevel() == logging.DEBUG:
             logging.info("Plotting results")
-            fig = pylab.figure()
+            fig, ax = pylab.subplots()
             fig.suptitle('cacking of a flat image: SplitPixel')
-            sp = fig.add_subplot(111)
-            sp.imshow(I, interpolation="nearest")
+            ax.imshow(I, interpolation="nearest")
             fig.show()
-            six.moves.input("Press enter to quit")
+            input("Press enter to quit")
         I[I == -1.0] = 1.0
         assert abs(I.min() - 1.0) < self.epsilon
         assert abs(I.max() - 1.0) < self.epsilon
 
     def test_splitBBox(self):
-        shape = (200, 201)
-        data = numpy.ones(shape, dtype="float64")
-        det = Detector(1e-4, 1e-4, max_shape=shape)
-        ai = AzimuthalIntegrator(0.1, 1e-2, 1e-2, detector=det)
-        I = ai.integrate2d(data, 256, 256, correctSolidAngle=False, dummy=-1.0,
+        I = self.ai.integrate2d(self.data, 256, 256, correctSolidAngle=False, dummy=-1.0,
                            unit="2th_deg", method='splitbbox')[0]
 
         if logger.getEffectiveLevel() == logging.DEBUG:
             logging.info("Plotting results")
-            fig = pylab.figure()
+            fig, ax = pylab.subplots()
             fig.suptitle('cacking of a flat image: SplitBBox')
-            sp = fig.add_subplot(111)
-            sp.imshow(I, interpolation="nearest")
+            ax.imshow(I, interpolation="nearest")
             fig.show()
-            six.moves.input("Press enter to quit")
+            input("Press enter to quit")
         I[I == -1.0] = 1.0
         assert abs(I.min() - 1.0) < self.epsilon
         assert abs(I.max() - 1.0) < self.epsilon
@@ -450,11 +454,11 @@ class TestSaxs(unittest.TestCase):
         ref1d = {}
         ref2d = {}
 
-        data = fabio.open(self.edfPilatus).data[:ai.detector.shape[0], :ai.detector.shape[1]]
+        data = fabio.open(self.edfPilatus).data[:ai.detector.shape[0],:ai.detector.shape[1]]
         for method in methods:
             logger.debug("TestSaxs.test_normalization_factor method= " + method)
-            ref1d[method + "_1"] = ai.integrate1d(copy.deepcopy(data), 100, method=method, error_model="poisson")
-            ref1d[method + "_10"] = ai.integrate1d(copy.deepcopy(data), 100, method=method, normalization_factor=10, error_model="poisson")
+            ref1d[method + "_1"] = ai.integrate1d_ng(copy.deepcopy(data), 100, method=method, error_model="poisson")
+            ref1d[method + "_10"] = ai.integrate1d_ng(copy.deepcopy(data), 100, method=method, normalization_factor=10, error_model="poisson")
             ratio_i = ref1d[method + "_1"].intensity.mean() / ref1d[method + "_10"].intensity.mean()
             ratio_s = ref1d[method + "_1"].sigma.mean() / ref1d[method + "_10"].sigma.mean()
 
@@ -488,9 +492,10 @@ class TestSaxs(unittest.TestCase):
         ai.setFit2D(2000, 870, 102.123456789)  # rational numbers are hell !
         ai.wavelength = 1e-10
         mask = img < 0
-        res_poisson = ai.integrate1d(img, 1000, mask=mask, error_model="poisson")
+        res_poisson = ai.integrate1d_ng(img, 1000, mask=mask, error_model="poisson")
         self.assertGreater(res_poisson.sigma.min(), 0, "Poisson error are positive")
-        res_azimuthal = ai.integrate1d(img, 1000, mask=mask, error_model="azimuthal")
+# TODO bug 1446 error-model "azimuthal" is not implemented in `integrate1d_ng`
+        res_azimuthal = ai.integrate1d_legacy(img, 1000, mask=mask, error_model="azimuthal")
         self.assertGreater(res_azimuthal.sigma.min(), 0, "Azimuthal error are positive")
 
 
@@ -502,21 +507,13 @@ class TestSetter(unittest.TestCase):
         self.rnd1 = numpy.random.random(shape).astype(numpy.float32)
         self.rnd2 = numpy.random.random(shape).astype(numpy.float32)
 
-        tmp_dir = os.path.join(UtilsTest.tempdir, self.id())
-        if not os.path.isdir(tmp_dir):
-            os.mkdir(tmp_dir)
-        self.tmp_dir = tmp_dir
-
-        fd, self.edf1 = tempfile.mkstemp(".edf", "testAI1", tmp_dir)
+        fd, self.edf1 = UtilsTest.tempfile(".edf", "testAI1", dir=__class__.__name__)
         os.close(fd)
-        fd, self.edf2 = tempfile.mkstemp(".edf", "testAI2", tmp_dir)
+        fd, self.edf2 = UtilsTest.tempfile(".edf", "testAI2", dir=__class__.__name__)
         os.close(fd)
         fabio.edfimage.edfimage(data=self.rnd1).write(self.edf1)
         fabio.edfimage.edfimage(data=self.rnd2).write(self.edf2)
 
-    def tearDown(self):
-        shutil.rmtree(self.tmp_dir)
-
     def test_flat(self):
         self.ai.set_flatfiles((self.edf1, self.edf2), method="mean")
         self.assertTrue(self.ai.flatfiles == "%s(%s,%s)" % ("mean", self.edf1, self.edf2), "flatfiles string is OK")
@@ -542,11 +539,12 @@ class TestIntergrationNextGeneration(unittest.TestCase):
 
         method = ("no", "histogram", "cython")
         cython = ai._integrate1d_ng(data, 100, method=method, error_model="poisson")
+
         self.assertEqual(cython.compute_engine, "pyFAI.ext.histogram.histogram1d_engine")
         self.assertEqual(str(cython.unit), "q_nm^-1")
         self.assertTrue(numpy.allclose(cython.radial, python.radial), "cython position are the same")
         self.assertTrue(numpy.allclose(cython.intensity, python.intensity), "cython intensities are the same")
-        self.assertTrue(numpy.allclose(cython.sigma, python.sigma), "cython errors are the same")
+        self.assertTrue(numpy.allclose(cython.sigma, python.sigma), f"cython errors are the same, aerr={(abs(python.sigma - cython.sigma)).max()}")
         self.assertTrue(numpy.allclose(cython.sum_signal, python.sum_signal), "cython sum_signal are the same")
         self.assertTrue(numpy.allclose(cython.sum_variance, python.sum_variance), "cython sum_variance are the same")
         self.assertTrue(numpy.allclose(cython.sum_normalization, python.sum_normalization), "cython sum_normalization are the same")
@@ -560,16 +558,81 @@ class TestIntergrationNextGeneration(unittest.TestCase):
         opencl = ai._integrate1d_ng(data, 100, method=method, error_model="poisson")
         self.assertEqual(opencl.compute_engine, "pyFAI.opencl.azim_hist.OCL_Histogram1d")
         self.assertEqual(str(opencl.unit), "q_nm^-1")
-
         self.assertTrue(numpy.allclose(opencl.radial, python.radial), "opencl position are the same")
         self.assertTrue(numpy.allclose(opencl.intensity, python.intensity), "opencl intensities are the same")
         self.assertTrue(numpy.allclose(opencl.sigma, python.sigma), "opencl errors are the same")
         self.assertTrue(numpy.allclose(opencl.sum_signal.sum(axis=-1), python.sum_signal), "opencl sum_signal are the same")
-        self.assertTrue(numpy.allclose(opencl.sum_variance.sum(axis=-1), python.sum_variance), "opencl sum_variance are the same")
+        self.assertTrue(numpy.allclose(opencl.sum_variance.sum(axis=-1), python.sum_variance),
+                        f"opencl sum_variance are the same {abs(opencl.sum_variance.sum(axis=-1) - python.sum_variance).max()}")
         self.assertTrue(numpy.allclose(opencl.sum_normalization.sum(axis=-1), python.sum_normalization), "opencl sum_normalization are the same")
         self.assertTrue(numpy.allclose(opencl.count, python.count), "opencl count are the same")
 
 
+class TestRange(unittest.TestCase):
+    """Test for reduced range in radia/azimuthal direction"""
+
+    @classmethod
+    def setUpClass(cls):
+        cls.img = fabio.open(UtilsTest.getimage("Pilatus1M.edf")).data
+        cls.ai = AzimuthalIntegrator.sload(UtilsTest.getimage("Pilatus1M.poni"))
+        cls.unit = "r_mm"
+        cls.azim_range = (-90, 90)
+        cls.rad_range = (10, 100)
+        cls.npt = 500
+        # cls.ref_medfilt = cls.ai.medfilt1d(cls.img, cls.npt, unit=cls.unit)
+        centerx = cls.ai.getFit2D()["centerX"]
+        cls.img[:, int(centerx - 1)] = 0
+
+    @classmethod
+    def tearDownClass(cls):
+        cls.unit = cls.azim_range = cls.rad_range = cls.ai = cls.img = None
+
+    def test_medfilt(self):
+        self.ai.reset()
+        res = self.ai.medfilt1d(self.img, self.npt, unit=self.unit, azimuth_range=self.azim_range, radial_range=self.rad_range)
+        self.assertGreaterEqual(res.radial.min(), min(self.rad_range))
+        self.assertLessEqual(res.radial.max(), max(self.rad_range))
+
+    def test_sigma_clip_legacy(self):
+        self.ai.reset()
+        res = self.ai._sigma_clip_legacy(self.img, self.npt, unit=self.unit, azimuth_range=self.azim_range, radial_range=self.rad_range)
+        self.assertGreaterEqual(res.radial.min(), min(self.rad_range))
+        self.assertLessEqual(res.radial.max(), max(self.rad_range))
+
+    def test_sigma_clip_ng(self):
+        self.ai.reset()
+
+        for case in ({"error_model":"azimuthal", "max_iter":3, "thres":0},
+                     {"error_model":"poisson", "max_iter":3, "thres":6}):
+            results = {}
+            for impl in ('python', 'cython', 'opencl'):
+                try:
+                    res = self.ai.sigma_clip_ng(self.img, self.npt, unit=self.unit,
+                                                azimuth_range=self.azim_range, radial_range=self.rad_range,
+                                                method=("no", "csr", impl),
+                                                **case)
+                except RuntimeError as err:
+                    logger.warning("got RuntimeError with impl %s: %s case: %s", impl, err, case)
+                    continue
+                else:
+                    results[impl] = res
+                self.assertGreaterEqual(res.radial.min(), min(self.rad_range), msg=f"impl: {impl}, case {case}")
+                self.assertLessEqual(res.radial.max(), max(self.rad_range), msg=f"impl: {impl}, case {case}")
+            ref = results['cython']
+            for what, tol in (("radial", 1e-8),
+                              ("intensity", 1e-6),
+                              ("sigma", 1e-6),
+                              ("sum_normalization", 1e-1),
+                              ("count", 1e-1)):
+                for impl in results:
+                    obt = results[impl]
+                    # print(what, obt.__getattribute__(what).max(),
+                    # abs(ref.__getattribute__(what) - obt.__getattribute__(what)).max(),
+                    # abs((ref.__getattribute__(what) - obt.__getattribute__(what)) / ref.__getattribute__(what)).max())
+                    self.assertTrue(numpy.allclose(obt.__getattribute__(what), ref.__getattribute__(what), atol=10, rtol=tol),
+                                    msg=f"Sigma clipping matches for impl {impl} on paramter {what}")
+
+
 def suite():
     loader = unittest.defaultTestLoader.loadTestsFromTestCase
     testsuite = unittest.TestSuite()
@@ -580,6 +643,7 @@ def suite():
     # testsuite.addTest(loader(TestAzimPilatus))
     testsuite.addTest(loader(TestSaxs))
     testsuite.addTest(loader(TestIntergrationNextGeneration))
+    testsuite.addTest(loader(TestRange))
     return testsuite
 
 
