@@ -26,7 +26,6 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-
 import json
 import os
 import fabio
@@ -64,6 +63,9 @@ class TestIntegrateApp(unittest.TestCase):
             self.monitor_key = None
             self.write_mode = None
 
+        def __repr__(self):
+            return "\n".join(["%s: %s" % (k, self.__getattribute__(k)) for k in dir(self) if "_" not in k])
+
     def get_path(self, filename):
         path = os.path.join(self.tempDir, filename)
         return path
@@ -73,7 +75,7 @@ class TestIntegrateApp(unittest.TestCase):
         if isinstance(data, list):
             image = fabio.edfimage.EdfImage(data=data[0], header=header)
             for d in data[1:]:
-                image.appendFrame(data=d, header=header)
+                image.append_frame(data=d, header=header)
         else:
             image = fabio.edfimage.EdfImage(data=data, header=header)
         image.save(path)
@@ -100,7 +102,7 @@ class TestIntegrateApp(unittest.TestCase):
 
     def create_h5_cube_file(self, filename, datapath, data, monitor=None):
         path = os.path.join(self.tempDir, filename)
-        with h5py.File(path) as h5:
+        with h5py.File(path, "w") as h5:
             h5[datapath] = numpy.array(data)
             if monitor is not None:
                 h5["/monitor/data"] = numpy.array(monitor)
@@ -192,7 +194,7 @@ class TestIntegrateApp(unittest.TestCase):
         with h5py.File(options.output, mode="r") as h5:
             self.assertIsNotNone(h5)
             result = h5["entry_0000/integrate/results/data"]
-            self.assertEquals(result.shape[0], 4)
+            self.assertEqual(result.shape[0], 4)
             for iframe in range(result.shape[0]):
                 numpy.testing.assert_almost_equal(result[iframe], expected, decimal=1)
 
@@ -207,7 +209,7 @@ class TestIntegrateApp(unittest.TestCase):
         self.assertTrue(os.path.exists(options.output))
         with h5py.File(options.output, mode="r") as h5:
             self.assertIsNotNone(h5)
-            self.assertEquals(h5["entry_0000/integrate/results/data"].shape[0], 2)
+            self.assertEqual(h5["entry_0000/integrate/results/data"].shape[0], 2)
 
     def test_multiframes_to_h5(self):
         options = self.Options()
@@ -219,7 +221,7 @@ class TestIntegrateApp(unittest.TestCase):
         self.assertTrue(os.path.exists(options.output))
         with h5py.File(options.output, mode="r") as h5:
             self.assertIsNotNone(h5)
-            self.assertEquals(h5["entry_0000/integrate/results/data"].shape[0], 2)
+            self.assertEqual(h5["entry_0000/integrate/results/data"].shape[0], 2)
 
     def test_multiframes_fileseries_to_h5(self):
         options = self.Options()
@@ -232,7 +234,7 @@ class TestIntegrateApp(unittest.TestCase):
         self.assertTrue(os.path.exists(options.output))
         with h5py.File(options.output, mode="r") as h5:
             self.assertIsNotNone(h5)
-            self.assertEquals(h5["entry_0000/integrate/results/data"].shape[0], 4)
+            self.assertEqual(h5["entry_0000/integrate/results/data"].shape[0], 4)
 
     def test_h5_to_h5(self):
         options = self.Options()
@@ -244,7 +246,7 @@ class TestIntegrateApp(unittest.TestCase):
         self.assertTrue(os.path.exists(options.output))
         with h5py.File(options.output, mode="r") as h5:
             self.assertIsNotNone(h5)
-            self.assertEquals(h5["entry_0000/integrate/results/data"].shape[0], 4)
+            self.assertEqual(h5["entry_0000/integrate/results/data"].shape[0], 4)
 
     def test_fileseries_to_h5_entry(self):
         options = self.Options()
@@ -258,7 +260,7 @@ class TestIntegrateApp(unittest.TestCase):
         self.assertTrue(os.path.exists(output_file))
         with h5py.File(output_file, mode="r") as h5:
             self.assertIsNotNone(h5)
-            self.assertEquals(h5["my/entry/integrate/results/data"].shape[0], 2)
+            self.assertEqual(h5["my/entry/integrate/results/data"].shape[0], 2)
 
     def test_write_h5_delete_mode(self):
         options = self.Options()
@@ -274,7 +276,7 @@ class TestIntegrateApp(unittest.TestCase):
         self.assertTrue(os.path.exists(options.output))
         with h5py.File(options.output, mode="r") as h5:
             self.assertIsNotNone(h5)
-            self.assertEquals(h5["/entry_0000/integrate/results/data"].shape[0], 1)
+            self.assertEqual(h5["/entry_0000/integrate/results/data"].shape[0], 1)
 
     def test_write_h5_overwrite_mode(self):
         options = self.Options()
@@ -291,8 +293,8 @@ class TestIntegrateApp(unittest.TestCase):
         self.assertTrue(os.path.exists(options.output))
         with h5py.File(options.output, mode="r") as h5:
             self.assertIsNotNone(h5)
-            self.assertEquals(h5["/entry_0000/integrate/results/data"].shape[0], 1)
-            self.assertEquals(h5["/my_entry"][()], 10)
+            self.assertEqual(h5["/entry_0000/integrate/results/data"].shape[0], 1)
+            self.assertEqual(h5["/my_entry"][()], 10)
 
     def test_write_h5_append_mode(self):
         options = self.Options()
@@ -308,8 +310,8 @@ class TestIntegrateApp(unittest.TestCase):
         self.assertTrue(os.path.exists(options.output))
         with h5py.File(options.output, mode="r") as h5:
             self.assertIsNotNone(h5)
-            self.assertEquals(h5["/entry_0000"][()], 10)
-            self.assertEquals(h5["/entry_0001/integrate/results/data"].shape[0], 1)
+            self.assertEqual(h5["/entry_0000"][()], 10)
+            self.assertEqual(h5["/entry_0001/integrate/results/data"].shape[0], 1)
 
     def test_write_h5_error_mode(self):
         options = self.Options()
@@ -326,7 +328,7 @@ class TestIntegrateApp(unittest.TestCase):
         self.assertTrue(os.path.exists(options.output))
         with h5py.File(options.output, mode="r") as h5:
             self.assertIsNotNone(h5)
-            self.assertEquals(h5["/entry_0000"][()], 10)
+            self.assertEqual(h5["/entry_0000"][()], 10)
             self.assertNotIn("/entry_0001", h5)
 
 
@@ -381,7 +383,7 @@ class TestProcess(unittest.TestCase):
 
     def test_process_numpy_2d(self):
         data = numpy.array([[0, 0], [0, 100], [0, 0]])
-        expected_result = [[5.6, 4.5], [41.8, 9.3]]
+        expected_result = [[7.5, 5.3], [50.1, 12.6]]
         expected_radial = [2.0, 2.0]
         expected_azimuthal = [-124.5, -124.2]
         params = {"do_2D": True,
@@ -400,7 +402,7 @@ class TestProcess(unittest.TestCase):
 
     def test_process_numpy_3d(self):
         data = numpy.array([[[0, 0], [0, 100], [0, 0]], [[0, 0], [0, 200], [0, 0]]])
-        expected_result = [[5.6, 4.5], [41.8, 9.3]]
+        expected_result = [[7.5, 5.3], [50.1, 12.6]]
         expected_radial = [2.0, 2.0]
         expected_azimuthal = [-124.5, -124.2]
         params = {"do_2D": True,
@@ -438,9 +440,9 @@ class TestProcess(unittest.TestCase):
         data1 = numpy.array([[0, 0], [0, 100], [0, 0]])
         data2 = numpy.array([[0, 0], [0, 200], [0, 0]])
         data = fabio.edfimage.EdfImage(data=data1)
-        data.appendFrame(data=data2)
+        data.append_frame(data=data2)
 
-        expected_result = [[5.6, 4.5], [41.8, 9.3]]
+        expected_result = [[7.5, 5.3], [50.1, 12.6]]
         expected_radial = [2.0, 2.0]
         expected_azimuthal = [-124.5, -124.2]
         params = {"do_2D": True,
