@@ -26,16 +26,13 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-
-from __future__ import division, print_function, absolute_import
-
 """Test suite for math utilities library"""
 
 __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "18/10/2019"
+__date__ = "16/10/2020"
 
 import unittest
 import numpy
@@ -68,11 +65,11 @@ class TestEdfMonitor(unittest.TestCase):
 
     def test_monitor(self):
         result = header_utils._get_monitor_value_from_edf(self.image, "mon1")
-        self.assertEquals(100, result)
+        self.assertEqual(100, result)
 
     def test_monitor_in_counter(self):
         result = header_utils._get_monitor_value_from_edf(self.image, "counter/mon3")
-        self.assertEquals(13, result)
+        self.assertEqual(13, result)
 
     def test_bad_monitor(self):
         self.assertRaises(header_utils.MonitorNotFound, header_utils._get_monitor_value_from_edf, self.image, "bad")
@@ -146,7 +143,7 @@ class TestHdf5Monitor(unittest.TestCase):
             for iframe in range(image.nframes):
                 frame = image.getframe(iframe)
                 result = header_utils.get_monitor_value(frame, monitor_key)
-                self.assertEquals(1.5, result)
+                self.assertEqual(1.5, result)
 
     def test_missing_monitor(self):
         monitor_key = "/header/bar/vector"
