@@ -28,14 +28,11 @@
 
 "Test suite for worker"
 
-from __future__ import absolute_import, division, print_function
-
 __author__ = "Valentin Valls"
 __contact__ = "valentin.valls@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "01/03/2019"
-
+__date__ = "30/06/2021"
 
 import unittest
 import logging
@@ -53,7 +50,6 @@ from ..containers import Integrate1dResult
 from ..containers import Integrate2dResult
 from . import utilstest
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -73,6 +69,8 @@ class AzimuthalIntegratorMocked():
             raise self._result
         return self._result
 
+    integrate1d_ng = integrate1d_legacy = integrate1d
+
     def integrate2d(self, **kargs):
         self._integrate2d_called += 1
         self._integrate2d_kargs = kargs
@@ -80,6 +78,8 @@ class AzimuthalIntegratorMocked():
             raise self._result
         return self._result
 
+    integrate2d_ng = integrate2d_legacy = integrate2d
+
 
 class MockedAiWriter():
 
@@ -110,80 +110,79 @@ class TestWorker(unittest.TestCase):
     def test_process_1d(self):
         ai_result = Integrate1dResult(numpy.array([0, 1]), numpy.array([2, 3]))
         ai = AzimuthalIntegratorMocked(result=ai_result)
-        worker = Worker(ai)
+        worker = Worker(ai, shapeOut=(1, 10))
         data = numpy.array([0])
         worker.unit = units.TTH_DEG
         worker.dummy = "b"
         worker.delta_dummy = "c"
-        worker.method = "d"
+        worker.method = "lut"
         worker.polarization_factor = "e"
         worker.correct_solid_angle = "f"
         worker.nbpt_rad = "g"
-        worker.nbpt_azim = 1
         worker.output = "numpy"
+        worker.update_processor()
         result = worker.process(data)
 
         # ai calls
-        self.assertEquals(ai._integrate1d_called, 1)
-        self.assertEquals(ai._integrate2d_called, 0)
+        self.assertEqual(ai._integrate1d_called, 1)
+        self.assertEqual(ai._integrate2d_called, 0)
         ai_args = ai._integrate1d_kargs
-        self.assertEquals(ai_args["unit"], worker.unit)
-        self.assertEquals(ai_args["dummy"], worker.dummy)
-        self.assertEquals(ai_args["delta_dummy"], worker.delta_dummy)
-        self.assertEquals(ai_args["method"], worker.method)
-        self.assertEquals(ai_args["polarization_factor"], worker.polarization_factor)
-        self.assertEquals(ai_args["safe"], True)
-        self.assertEquals(ai_args["data"], data)
-        self.assertEquals(ai_args["correctSolidAngle"], worker.correct_solid_angle)
-        self.assertEquals(ai_args["npt"], worker.nbpt_rad)
+        self.assertEqual(ai_args["unit"], worker.unit)
+        self.assertEqual(ai_args["dummy"], worker.dummy)
+        self.assertEqual(ai_args["delta_dummy"], worker.delta_dummy)
+        self.assertTrue(worker.method in str(ai_args["method"]).lower())
+        self.assertEqual(ai_args["polarization_factor"], worker.polarization_factor)
+        self.assertEqual(ai_args["safe"], True)
+        self.assertEqual(ai_args["data"], data)
+        self.assertEqual(ai_args["correctSolidAngle"], worker.correct_solid_angle)
+        self.assertEqual(ai_args["npt"], worker.nbpt_rad)
 
         # result
-        print(worker.radial)
-        self.assertEquals(result.tolist(), [[0, 2], [1, 3]])
-        self.assertEquals(worker.radial.tolist(), [0, 1])
-        self.assertEquals(worker.azimuthal, None)
+        self.assertEqual(result.tolist(), [[0, 2], [1, 3]])
+        self.assertEqual(worker.radial.tolist(), [0, 1])
+        self.assertEqual(worker.azimuthal, None)
 
     def test_process_2d(self):
         ai_result = Integrate2dResult(numpy.array([0]), numpy.array([1]), numpy.array([2]))
         ai = AzimuthalIntegratorMocked(result=ai_result)
-        worker = Worker(ai)
+        worker = Worker(ai, shapeOut=(10, 10))
         data = numpy.array([0])
         worker.unit = units.TTH_RAD
         worker.dummy = "b"
         worker.delta_dummy = "c"
-        worker.method = "d"
+        worker.method = "lut"
         worker.polarization_factor = "e"
         worker.correct_solid_angle = "f"
         worker.nbpt_rad = "g"
-        worker.nbpt_azim = 2
         worker.output = "numpy"
+        worker.update_processor()
         result = worker.process(data)
 
         # ai calls
         self.assertEqual(ai._integrate1d_called, 0)
         self.assertEqual(ai._integrate2d_called, 1)
         ai_args = ai._integrate2d_kargs
-        self.assertEquals(ai_args["unit"], worker.unit)
-        self.assertEquals(ai_args["dummy"], worker.dummy)
-        self.assertEquals(ai_args["delta_dummy"], worker.delta_dummy)
-        self.assertEquals(ai_args["method"], worker.method)
-        self.assertEquals(ai_args["polarization_factor"], worker.polarization_factor)
-        self.assertEquals(ai_args["safe"], True)
-        self.assertEquals(ai_args["data"], data)
-        self.assertEquals(ai_args["correctSolidAngle"], worker.correct_solid_angle)
-        self.assertEquals(ai_args["npt_rad"], worker.nbpt_rad)
-        self.assertEquals(ai_args["npt_azim"], worker.nbpt_azim)
+        self.assertEqual(ai_args["unit"], worker.unit)
+        self.assertEqual(ai_args["dummy"], worker.dummy)
+        self.assertEqual(ai_args["delta_dummy"], worker.delta_dummy)
+        self.assertTrue(worker.method in str(ai_args["method"]).lower())
+        self.assertEqual(ai_args["polarization_factor"], worker.polarization_factor)
+        self.assertEqual(ai_args["safe"], True)
+        self.assertEqual(ai_args["data"], data)
+        self.assertEqual(ai_args["correctSolidAngle"], worker.correct_solid_angle)
+        self.assertEqual(ai_args["npt_rad"], worker.nbpt_rad)
+        self.assertEqual(ai_args["npt_azim"], worker.nbpt_azim)
 
         # result
-        self.assertEquals(result.tolist(), [0])
-        self.assertEquals(worker.radial.tolist(), [1])
-        self.assertEquals(worker.azimuthal.tolist(), [2])
+        self.assertEqual(result.tolist(), [0])
+        self.assertEqual(worker.radial.tolist(), [1])
+        self.assertEqual(worker.azimuthal.tolist(), [2])
 
     def test_1d_writer(self):
         ai_result = Integrate1dResult(numpy.array([0]), numpy.array([1]))
         ai = AzimuthalIntegratorMocked(result=ai_result)
         data = numpy.array([0])
-        worker = Worker(ai)
+        worker = Worker(ai, shapeOut=(1, 10))
         worker.unit = units.TTH_DEG
         worker.dummy = "b"
         worker.delta_dummy = "c"
@@ -191,21 +190,21 @@ class TestWorker(unittest.TestCase):
         worker.polarization_factor = "e"
         worker.correct_solid_angle = "f"
         worker.nbpt_rad = "g"
-        worker.nbpt_azim = 1
+        # worker.nbpt_azim = 1
         worker.output = "numpy"
 
         writer = MockedAiWriter()
         _result = worker.process(data, writer=writer)
 
-        self.assertEquals(writer._write_called, 1)
-        self.assertEquals(writer._close_called, 0)
+        self.assertEqual(writer._write_called, 1)
+        self.assertEqual(writer._close_called, 0)
         self.assertIs(writer._write_kargs["data"], ai_result)
 
     def test_2d_writer(self):
         ai_result = Integrate2dResult(numpy.array([0]), numpy.array([1]), numpy.array([2]))
         ai = AzimuthalIntegratorMocked(result=ai_result)
         data = numpy.array([0])
-        worker = Worker(ai)
+        worker = Worker(ai, shapeOut=(1, 10))
         worker.unit = units.TTH_DEG
         worker.dummy = "b"
         worker.delta_dummy = "c"
@@ -213,21 +212,21 @@ class TestWorker(unittest.TestCase):
         worker.polarization_factor = "e"
         worker.correct_solid_angle = "f"
         worker.nbpt_rad = "g"
-        worker.nbpt_azim = 1
+        # worker.nbpt_azim = 1
         worker.output = "numpy"
 
         writer = MockedAiWriter()
         _result = worker.process(data, writer=writer)
 
-        self.assertEquals(writer._write_called, 1)
-        self.assertEquals(writer._close_called, 0)
+        self.assertEqual(writer._write_called, 1)
+        self.assertEqual(writer._close_called, 0)
         self.assertIs(writer._write_kargs["data"], ai_result)
 
     def test_process_exception(self):
         ai = AzimuthalIntegratorMocked(result=Exception("Out of memory"))
         worker = Worker(ai)
         data = numpy.array([0])
-        worker.nbpt_azim = 2
+        # worker.nbpt_azim = 2
         try:
             worker.process(data)
         except Exception:
@@ -236,21 +235,21 @@ class TestWorker(unittest.TestCase):
     def test_process_poisson(self):
         ai_result = Integrate1dResult(numpy.array([0]), numpy.array([1]))
         ai = AzimuthalIntegratorMocked(result=ai_result)
-        worker = Worker(ai)
+        worker = Worker(ai, shapeOut=(1, 10))
         data = numpy.array([0])
-        worker.nbpt_azim = 1
+        # worker.nbpt_azim = 1
         worker.do_poisson = True
         worker.process(data)
         self.assertIn("error_model", ai._integrate1d_kargs)
-        self.assertEquals(ai._integrate1d_kargs["error_model"], "poisson")
+        self.assertEqual(ai._integrate1d_kargs["error_model"], "poisson")
 
     def test_process_no_output(self):
         ai_result = Integrate1dResult(numpy.array([0]), numpy.array([1]))
         ai = AzimuthalIntegratorMocked(result=ai_result)
-        worker = Worker(ai)
+        worker = Worker(ai, shapeOut=(1, 10))
         data = numpy.array([0])
         worker.output = None
-        worker.nbpt_azim = 1
+        # worker.nbpt_azim = 1
         worker.do_poisson = True
         result = worker.process(data)
         self.assertIsNone(result)
@@ -305,7 +304,6 @@ class TestWorkerConfig(unittest.TestCase):
     def setUpClass(cls):
         cls.directory = os.path.join(utilstest.test_options.tempdir, cls.__name__)
         os.makedirs(cls.directory)
-        print(cls.directory)
 
         cls.a = os.path.join(cls.directory, "a.npy")
         cls.b = os.path.join(cls.directory, "b.npy")
