@@ -4,7 +4,7 @@
 #    Project: Azimuthal integration
 #             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2015-2018 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2015-2022 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -78,14 +78,11 @@ Here are the valid keys:
 - "method"
 """
 
-
-from __future__ import with_statement, print_function, division
-
-__author__ = "Jerome Kieffer"
+__author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "24/01/2020"
+__date__ = "01/02/2022"
 __status__ = "development"
 
 import threading
@@ -93,18 +90,27 @@ import os.path
 import logging
 import json
 import numpy
+from collections import OrderedDict
 
 logger = logging.getLogger(__name__)
 
-from .third_party import six
 from . import average
 from . import method_registry
 from .azimuthalIntegrator import AzimuthalIntegrator
+from pyFAI.method_registry import IntegrationMethod
 from .distortion import Distortion
 from . import units
 from .io import integration_config
 import pyFAI.io.image
 from .engines.preproc import preproc as preproc_numpy
+try:
+    import numexpr
+except ImportError as err:
+    logger.warning("Unable to import Cython version of preproc: %s", err)
+    USE_NUMEXPR = False
+else:
+    USE_NUMEXPR = True
+
 try:
     from .ext.preproc import preproc
 except ImportError as err:
@@ -142,22 +148,10 @@ def _init_ai(ai, config, consume_keys=False, read_maps=True):
     if not consume_keys:
         config = dict(config)
 
-    # Geometry
-    for key in ("dist", "poni1", "poni2", "rot1", "rot2", "rot3"):
-        value = config.pop(key, None)
-        if value is not None:
-            ai.__setattr__(key, value)
-    wavelength = config.pop("wavelength", None)
-    if wavelength:
-        if wavelength <= 0 or wavelength > 1e-6:
-            logger.warning("Wavelength is in meter... unlikely value %s", wavelength)
-        ai.wavelength = wavelength
-
-    # Detector
-    reader = integration_config.ConfigurationReader(config)
-    detector = reader.pop_detector()
-    if detector is not None:
-        ai.detector = detector
+#   #This sets only what is part of the poni-file
+    config_reader = integration_config.ConfigurationReader(config)
+    poni = config_reader.pop_ponifile()
+    ai._init_from_poni(poni)
 
     value = config.pop("chi_discontinuity_at_0", False)
     if value:
@@ -197,7 +191,7 @@ def _normalize_filenames(filenames):
         return []
     if isinstance(filenames, list):
         return filenames
-    if isinstance(filenames, six.string_types):
+    if isinstance(filenames, (str,)):
         # It's a single filename
         return [filenames]
     raise TypeError("Unsupported type %s for a list of filenames" % type(filenames))
@@ -219,14 +213,22 @@ def _reduce_images(filenames, method="mean"):
 
 
 class Worker(object):
+
     def __init__(self, azimuthalIntegrator=None,
                  shapeIn=(2048, 2048), shapeOut=(360, 500),
-                 unit="r_mm", dummy=None, delta_dummy=None):
+                 unit="r_mm", dummy=None, delta_dummy=None,
+                 method=("bbox", "csr", "cython"),
+                 integrator_name=None, extra_options=None):
         """
         :param AzimuthalIntegrator azimuthalIntegrator: An AzimuthalIntegrator instance
-        :param shapeIn: image size in input
-        :param shapeOut: Integrated size: can be (1,2000) for 1D integration
-        :param unit: can be "2th_deg, r_mm or q_nm^-1 ...
+        :param tuple shapeIn: image size in input
+        :param tuple shapeOut: Integrated size: can be (1,2000) for 1D integration
+        :param str unit: can be "2th_deg, r_mm or q_nm^-1 ...
+        :param float dummy: the value making invalid pixels
+        :param float delta_dummy: the precision for dummy values
+        :param method: integration method: str like "csr" or tuple ("bbox", "csr", "cython") or IntegrationMethod instance.
+        :param str integrator_name: Offers an alternative to "integrate1d" like "sigma_clip_ng"
+        :param dict extra_options: extra kwargs for the integrator (like {"max_iter":3, "thres":0, "error_model": "azimuthal"} for sigma-clipping)
         """
         self._sem = threading.Semaphore()
         if azimuthalIntegrator is None:
@@ -234,6 +236,11 @@ class Worker(object):
         else:
             self.ai = azimuthalIntegrator
         self._normalization_factor = None  # Value of the monitor: divides the intensity by this value for normalization
+        self.integrator_name = integrator_name
+        self._processor = None
+        self._nbpt_azim = None
+        self.method = method
+        self._method = None
         self.nbpt_azim, self.nbpt_rad = shapeOut
         self._unit = units.to_unit(unit)
         self.polarization_factor = None
@@ -246,15 +253,16 @@ class Worker(object):
         self.subdir = ""
         self.extension = None
         self.do_poisson = None
+        self.do_azimuthal_error = None
         self.needs_reset = True
         self.output = "numpy"  # exports as numpy array by default
         self.shape = shapeIn
-        self.method = "csr"
         self.radial = None
         self.azimuthal = None
         self.radial_range = None
         self.azimuth_range = None
         self.safe = True
+        self.extra_options = {} if extra_options is None else extra_options.copy()
 
     def __repr__(self):
         """
@@ -279,6 +287,26 @@ class Worker(object):
     def do_2D(self):
         return self.nbpt_azim > 1
 
+    def update_processor(self, integrator_name=None):
+        if integrator_name is None:
+            integrator_name = self.integrator_name
+        if integrator_name is None:
+            if self.do_2D():
+                integrator_name = "integrate2d"
+            else:
+                integrator_name = "integrate1d"
+        self._processor = self.ai.__getattribute__(integrator_name)
+        self._method = IntegrationMethod.select_one_available(self.method, dim=2 if self.do_2D() else 1)
+
+    @property
+    def nbpt_azim(self):
+        return self._nbpt_azim
+
+    @nbpt_azim.setter
+    def nbpt_azim(self, value):
+        self._nbpt_azim = value
+        self.update_processor()
+
     def reset(self):
         """
         this is just to force the integrator to initialize
@@ -302,7 +330,7 @@ class Worker(object):
 
     def process(self, data, variance=None, normalization_factor=1.0, writer=None, metadata=None):
         """
-        Process a frame
+        Process one frame
         #TODO:
         dark, flat, sa are missing
 
@@ -312,17 +340,21 @@ class Worker(object):
 
         with self._sem:
             monitor = self._normalization_factor * normalization_factor if self._normalization_factor else normalization_factor
-        kwarg = {"unit": self.unit,
-                 "dummy": self.dummy,
-                 "delta_dummy": self.delta_dummy,
-                 "method": self.method,
-                 "polarization_factor": self.polarization_factor,
-                 # "filename": None,
-                 "data": data,
-                 "correctSolidAngle": self.correct_solid_angle,
-                 "safe": self.safe,
-                 "variance": variance
-                 }
+        kwarg = self.extra_options.copy()
+        kwarg["unit"] = self.unit
+        kwarg["dummy"] = self.dummy
+        kwarg["delta_dummy"] = self.delta_dummy
+        kwarg["method"] = self._method
+        kwarg["polarization_factor"] = self.polarization_factor
+        kwarg["data"] = data
+        kwarg["correctSolidAngle"] = self.correct_solid_angle
+        kwarg["safe"] = self.safe
+        kwarg["variance"] = variance
+
+        if self.do_poisson:
+            kwarg["error_model"] = "poisson"
+        elif self.do_azimuthal_error:
+            kwarg["error_model"] = "azimuth"
 
         if metadata is not None:
             kwarg["metadata"] = metadata
@@ -333,18 +365,8 @@ class Worker(object):
         if self.do_2D():
             kwarg["npt_rad"] = self.nbpt_rad
             kwarg["npt_azim"] = self.nbpt_azim
-            # if "filename" in kwarg:
-            #    if self.extension:
-            #        kwarg["filename"] += self.extension
-            #    else:
-            #        kwarg["filename"] += ".azim"
         else:
             kwarg["npt"] = self.nbpt_rad
-            # if "filename" in kwarg:
-            #    if self.extension:
-            #        kwarg["filename"] += self.extension
-            #    else:
-            #        kwarg["filename"] += ".xy"
         kwarg["error_model"] = self.error_model
 
         if self.radial_range is not None:
@@ -355,15 +377,14 @@ class Worker(object):
 
         error = None
         try:
+            integrated_result = self._processor(**kwarg)
             if self.do_2D():
-                integrated_result = self.ai.integrate2d(**kwarg)
                 self.radial = integrated_result.radial
                 self.azimuthal = integrated_result.azimuthal
                 result = integrated_result.intensity
                 if variance is not None:
                     error = integrated_result.sigma
             else:
-                integrated_result = self.ai.integrate1d(**kwarg)
                 self.radial = integrated_result.radial
                 self.azimuthal = None
                 result = numpy.vstack(integrated_result).T
@@ -412,13 +433,15 @@ class Worker(object):
     def set_dark_current_file(self, imagefile):
         self.ai.detector.set_darkcurrent(_reduce_images(imagefile))
         self.dark_current_image = imagefile
+
     setDarkcurrentFile = set_dark_current_file
-    
+
     def set_flat_field_file(self, imagefile):
         self.ai.detector.set_flatfield(_reduce_images(imagefile))
         self.flat_field_image = imagefile
+
     setFlatfieldFile = set_flat_field_file
-    
+
     def set_config(self, config, consume_keys=False):
         """
         Configure the working from the dictionary.
@@ -443,7 +466,7 @@ class Worker(object):
             except Exception as error:
                 logger.error("Unable to load mask file %s, error %s", filename, error)
             else:
-                self.ai.mask = data
+                self.ai.detector.mask = data
 
         # Do it here while we have to store metadata
         filename = config.pop("dark_current", "")
@@ -510,8 +533,8 @@ class Worker(object):
         value = config.pop("do_solid_angle", True)
         self.correct_solid_angle = bool(value)
 
-        self.dummy = config.pop("delta_dummy", None)
-        self.delta_dummy = config.pop("val_dummy", None)
+        self.dummy = config.pop("val_dummy", None)
+        self.delta_dummy = config.pop("delta_dummy", None)
         apply_values = config.pop("do_dummy", True)
         if not apply_values:
             self.dummy, self.delta_dummy = None, None
@@ -520,7 +543,7 @@ class Worker(object):
 
         if "monitor_name" in config:
             logger.warning("Monitor name defined but unsupported by the worker.")
-
+        self.update_processor()
         logger.info(self.ai.__repr__())
         self.reset()
         # For now we do not calculate the LUT as the size of the input image is unknown
@@ -530,6 +553,7 @@ class Worker(object):
 
     def get_unit(self):
         return self._unit
+
     unit = property(get_unit, set_unit)
 
     def set_error_model(self, value):
@@ -539,10 +563,12 @@ class Worker(object):
             self.do_poisson = False
         else:
             raise RuntimeError("Unsupported error model '%s'" % value)
+
     def get_error_model(self):
         if self.do_poisson:
             return "poisson"
         return None
+
     error_model = property(get_error_model, set_error_model)
 
     def get_config(self):
@@ -550,32 +576,33 @@ class Worker(object):
 
         FIXME: The returned dictionary is not exhaustive.
         """
-        config = {"unit": str(self.unit)}
+        config = OrderedDict()
+        config["unit"] = str(self.unit)
         for key in ["dist", "poni1", "poni2", "rot1", "rot3", "rot2", "pixel1", "pixel2", "splineFile", "wavelength"]:
             try:
                 config[key] = self.ai.__getattribute__(key)
-            except:
+            except Exception:
                 pass
         for key in ["nbpt_azim", "nbpt_rad", "polarization_factor", "dummy", "delta_dummy",
                     "correct_solid_angle", "dark_current_image", "flat_field_image",
                     "mask_image", "do_poisson", "shape", "method"]:
             try:
                 config[key] = self.__getattribute__(key)
-            except:
+            except Exception:
                 pass
-        
+
         for key in ["azimuth_range", "radial_range"]:
             try:
                 value = self.__getattribute__(key)
-            except:
+            except Exception:
                 pass
             else:
                 if value is not None:
-                    config["do_"+key] = True
-                    config[key+"_min"] = min(value)
-                    config[key+"_max"] = max(value)
+                    config["do_" + key] = True
+                    config[key + "_min"] = min(value)
+                    config[key + "_max"] = max(value)
                 else:
-                    config["do_"+key] = False
+                    config["do_" + key] = False
 
         return config
 
@@ -590,8 +617,9 @@ class Worker(object):
         else:
             config = json.loads(json_file)
         self.set_config(config)
+
     setJsonConfig = set_json_config
-    
+
     def save_config(self, filename=None):
         """Save the configuration as a JSON file"""
         if not filename:
@@ -616,9 +644,11 @@ class Worker(object):
     def get_normalization_factor(self):
         with self._sem:
             return self._normalization_factor
+
     def set_normalization_factor(self, value):
         with self._sem:
             self._normalization_factor = value
+
     normalization_factor = property(get_normalization_factor, set_normalization_factor)
 
     def set_method(self, method="csr"):
@@ -628,7 +658,7 @@ class Worker(object):
             method = method_registry.Method(dim, "*", "*", "*", target=None)
         elif isinstance(method, method_registry.Method):
             method = method.fixed(dim=dim)
-        elif isinstance(method, six.string_types):
+        elif isinstance(method, (str,)):
             method = method_registry.Method.parsed(method)
             method = method.fixed(dim=dim)
         elif isinstance(method, (list, tuple)):
@@ -647,6 +677,7 @@ class PixelwiseWorker(object):
     """
     Simple worker doing dark, flat, solid angle and polarization correction
     """
+
     def __init__(self, dark=None, flat=None, solidangle=None, polarization=None,
                  mask=None, dummy=None, delta_dummy=None, device=None,
                  empty=None, dtype="float32"):
@@ -736,15 +767,18 @@ class DistortionWorker(object):
     """
     Simple worker doing dark, flat, solid angle and polarization correction
     """
+
     def __init__(self, detector=None, dark=None, flat=None, solidangle=None, polarization=None,
-                 mask=None, dummy=None, delta_dummy=None, device=None):
+                 mask=None, dummy=None, delta_dummy=None, method="LUT", device=None):
         """Constructor of the worker
         :param dark: array
         :param flat: array
         :param solidangle: solid-angle array
         :param polarization: numpy array with 2D polarization corrections
+        :param dummy: value for bad pixels
+        :param delta_dummy: precision for dummies
+        :param method: LUT or CSR for the correction
         :param device: Used to influance OpenCL behavour: can be "cpu", "GPU", "Acc" or even an OpenCL context
-
         """
 
         self.ctx = None
@@ -767,47 +801,73 @@ class DistortionWorker(object):
 
         if mask is None:
             self.mask = False
+            mask = numpy.zeros(detector.shape, dtype=bool)
         elif mask.min() < 0 and mask.max() == 0:  # 0 is valid, <0 is invalid
-            self.mask = (mask < 0)
+            mask = self.mask = (mask < 0)
         else:
-            self.mask = mask.astype(bool)
+            mask = self.mask = mask.astype(bool)
 
         self.dummy = dummy
         self.delta_dummy = delta_dummy
-        if device is not None:
-            logger.warning("GPU is not yet implemented")
 
-        if detector is None:
-            self.distortion = None
+        if detector is not None:
+            self.distortion = Distortion(detector, method=method, device=device,
+                                     mask=mask, empty=self.dummy or 0)
+            self.distortion.reset(prepare=True)  # enfoce initization
         else:
-            self.distortion = Distortion(detector, method="LUT", device=device,
-                                         mask=self.mask, empty=self.dummy or 0)
+            self.distortion = None
 
-    def process(self, data, variance=None,
+    def process(self,
+                data,
+                variance=None,
                 normalization_factor=1.0):
         """
         Process the data and apply a normalization factor
         :param data: input data
         :param variance: the variance associated to the data
         :param normalization: normalization factor
-        :return: processed data
+        :return: processed data as either an array (data) or two (data, error)
         """
-        proc_data = preproc(data,
-                            variance=variance,
-                            dark=self.dark,
-                            flat=self.flat,
-                            solidangle=self.solidangle,
-                            polarization=self.polarization,
-                            absorption=None,
-                            mask=self.mask,
-                            dummy=self.dummy,
-                            delta_dummy=self.delta_dummy,
-                            normalization_factor=normalization_factor,
-                            empty=None)
-
         if self.distortion is not None:
-            return self.distortion.correct(proc_data, self.dummy, self.delta_dummy)
+            return self.distortion.correct_ng(data,
+                                              variance=variance,
+                                              dark=self.dark,
+                                              flat=self.flat,
+                                              solidangle=self.solidangle,
+                                              polarization=self.polarization,
+                                              dummy=self.dummy,
+                                              delta_dummy=self.delta_dummy,
+                                              normalization_factor=normalization_factor)
         else:
-            return data
+            proc_data = preproc(data,
+                                variance=variance,
+                                dark=self.dark,
+                                flat=self.flat,
+                                solidangle=self.solidangle,
+                                polarization=self.polarization,
+                                absorption=None,
+                                mask=self.mask,
+                                dummy=self.dummy,
+                                delta_dummy=self.delta_dummy,
+                                normalization_factor=normalization_factor,
+                                empty=None)
+            if variance is not None:
+                pp_signal = proc_data[..., 0]
+                pp_variance = proc_data[..., 1]
+                pp_normalisation = proc_data[..., 2]
+                if numexpr is None:
+                    # Cheap, muthithreaded way:
+                    res_signal = numexpr.evaluate("where(pp_normalisation==0.0, 0.0, pp_signal / pp_normalisation)")
+                    res_error = numexpr.evaluate("where(pp_normalisation==0.0, 0.0, sqrt(pp_variance) / abs(pp_normalisation))")
+                else:
+                    # Take the numpy road:
+                    res_signal = numpy.zeros_like(pp_signal)
+                    res_error = numpy.zeros_like(pp_signal)
+                    msk = numpy.where(pp_normalisation != 0)
+                    res_signal[msk] = pp_signal[msk] / pp_normalisation[msk]
+                    res_error[msk] = numpy.sqrt(pp_variance[msk]) / abs(pp_normalisation[msk])
+                return res_signal, res_error
+            else:
+                return proc_data
 
     __call__ = process
